-------------------
-- Panel widget --
-------------------

InfoPanel = class(Widget)

function InfoPanel:init()
    self.ignoreFocus = true
    self.title = ""
    self.text =  {}
    self.visible = false
    self.dh = 100
    self.dw = 100
end

function InfoPanel:draw(gc)
    if self.visible then
        drawDialog(gc, self.title, self.x, self.y, self.w, 30 + #self.text*20)
        for i, text in pairs(self.text) do
            gc:drawString(text, self.x+4, self.y + 10 + 15*i, "top")
        end
    end
end

EditComponent = class(Dialog)

function EditComponent:init(element, x, y, w, h)
    Dialog.init(self, "Edit " .. element.kind, x, y, w, h)
    self.component = element.component
    self.parameter = element.parameter
    self.okayButton = sButton("Okay")
    self.okayButton.action = function()
        remove_screen(self)
    end
    self.nameLabel = sLabel("Name")
    self.valueLabel = sLabel(element.measure)
    self.nameInput = sInput()
    self.valueInput = sInput()
    self.nameInput.value = self.component.name
    self.valueInput.value = tostring(self.component[self.parameter].coeff)
    self.valueInput.number = true
    local units = {}
    for i, v in ipairs(Unit.prefixes) do
        units[i] = v .. self.component.unit
    end
    self.units = sDropdown(units)
		self.units.value = self.component[self.parameter]:unitStr()

    self:appendWidget(self.nameLabel, 4, 34)
    self:appendWidget(self.valueLabel, 4, 60)
    self:appendWidget(self.nameInput, 80, 34)
    self:appendWidget(self.valueInput, 80, 60)
    self:appendWidget(self.units, -4, 60)
    self:appendWidget(self.okayButton, -4, -4)
    self.valueInput:giveFocus()
end

function EditComponent:removed()
    self.component.name = self.nameInput.value:sub(0, 5)
    local prefix = string.sub(self.units.value, #self.units.value - #self.component.unit -  1, #self.units.value - #self.component.unit)
    self.component[self.parameter] = Unit(tonumber(self.valueInput.value), prefix, self.component.unit)
end


InfoWindow = class(Dialog)

function InfoWindow:init(title, lines, x, y, w, h)
    Dialog.init(self, title, x, y, w, h)
    self.okayButton = sButton("Okay")
    self.okayButton.action = function()
        remove_screen(self)
    end
    local longest = 0
    for i, v in pairs(lines) do
        local label = sLabel(v)
        self:appendWidget(label, 4, 10 + 15*i)
        longest = math.max(longest, #v)
    end
    -- self.h = 10 + 20*#lines
    -- self.w = 8*longest
    -- self.x = platform.window:width()/2 - self.w/2
    -- self.y = platform.window:height()/2 - self.h/2
    self.nameLabel = sLabel("Name")
    self:appendWidget(self.okayButton, -4, -4)
    self.okayButton:giveFocus()
end
