
function uCol(col)
		return col[1] or 0, col[2] or 0, col[3] or 0
end

function textLim(gc, text, max)
		local ttext, out = "",""
		local width	= gc:getStringWidth(text)
		if width<max then
				return text, width
		else
				for i=1, #text do
						ttext	= text:usub(1, i)
						if gc:getStringWidth(ttext .. "..")>max then
								break
						end
						out = ttext
				end
				return out .. "..", gc:getStringWidth(out .. "..")
		end
end


------------------------------------------------------------------
--                        Widget  Class                         --
------------------------------------------------------------------

Widget	=	class(Screen)

function Widget:init()
		self.dw	=	10
		self.dh	=	10

		self:ext()
end

function Widget:setSize(w, h)
		self.ww	= w or self.ww
		self.hh	= h or self.hh
end

function Widget:setPos(x, y)
		self.xx	= x or self.xx
		self.yy	= y or self.yy
end

function Widget:size(n)
		if n then return end
		self.w	=	math.floor(Pr(self.ww, self.dw, self.parent.w, 0)+.5)
		self.h	=	math.floor(Pr(self.hh, self.dh, self.parent.h, 0)+.5)

		self.rx	=	math.floor(Pr(self.xx, 0, self.parent.w, self.w)+.5)
		self.ry	=	math.floor(Pr(self.yy, 0, self.parent.h, self.h)+.5)
		self.x	=	self.parent.x + self.rx
		self.y	=	self.parent.y + self.ry
end

function Widget:giveFocus()
		if self.parent.focus~=0 then
				local veto	= self.parent:stealFocus()
				if veto == -1 then
						return -1
				end
		end

		self.hasFocus	=	true
		self.parent.focus	=	self.pid
		self:getFocus()
end

function Widget:getFocus() end
function Widget:loseFocus() end
function Widget:clearKey() 	end

function Widget:enterKey()
		self.parent:switchFocus(1)
end
function Widget:arrowKey(arrow)
		if arrow=="up" then
				self.parent:switchFocus(self.focusUp or -1)
		elseif arrow=="down"  then
				self.parent:switchFocus(self.focusDown or 1)
		elseif arrow=="left" then
				self.parent:switchFocus(self.focusLeft or -1)
		elseif arrow=="right"  then
				self.parent:switchFocus(self.focusRight or 1)
		end
end


WWidget	= addExtension(Widget, WidgetManager)


------------------------------------------------------------------
--                        Sample Widget                         --
------------------------------------------------------------------

-- First, create a new class based on Widget
box	=	class(Widget)

-- Init. You should define self.dh and self.dw, in case the user doesn't supply correct width/height values.
-- self.ww and self.hh can be a number or a string. If it's a number, the width will be that amount of pixels.
-- If it's a string, it will be interpreted as % of the parent screen size.
-- These values will be used to calculate self.w and self.h (don't write to this, it will overwritten everytime the widget get's painted)
-- self.xx and self.yy will be set when appending the widget to a screen. This value support the same % method (in a string)
-- They will be used to calculate self.x and self.h
function box:init(ww,hh,t)
		self.dh	= 10
		self.dw	= 10
		self.ww	= ww
		self.hh	= hh
		self.t	= t
end

-- Paint. Here you can paint your widget stuff
-- Variable you can use:
-- self.x, self.y	: numbers, x and y coordinates of the widget relative to screen. So it's the actual pixel position on the screen.
-- self.w, self.h	: numbers, w and h of widget
-- many others

function box:paint(gc)
		gc:setColorRGB(0,0,0)

		-- Do I have focus?
		if self.hasFocus then
			-- Yes, draw a filled black square
				gc:fillRect(self.x, self.y, self.w, self.h)
		else
			-- No, draw only the outline
				gc:drawRect(self.x, self.y, self.w, self.h)
		end

		gc:setColorRGB(128,128,128)
		if self.t then
				gc:drawString(self.t,self.x+2,self.y+2,"top")
		end
end


------------------------------------------------------------------
--                         Input Widget                         --
------------------------------------------------------------------


sInput	=	class(Widget)

function sInput:init()
		self.dw	=	100
		self.dh	=	20

		self.value	=	""
		self.bgcolor	=	{255,255,255}
		self.disabledcolor	= {128,128,128}
		self.font	=	{"sansserif", "r", 10}
		self.disabled	= false
end

function sInput:paint(gc)
		self.gc	=	gc
		local x	=	self.x
		local y = 	self.y

		gc:setPen("thin")
		gc:setFont(uCol(self.font))
		gc:setColorRGB(uCol(self.bgcolor))
		gc:fillRect(x, y, self.w, self.h)

		gc:setColorRGB(0,0,0)
		gc:drawRect(x, y, self.w, self.h)

		if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:drawRect(x-1, y-1, self.w+2, self.h+2)
        gc:setColorRGB(0, 0, 0)
    end

		local text	=	self.value
		local	p	=	0


		gc_clipRect(gc, "subset", x, y, self.w, self.h)

		--[[
		while true do
			if p==#self.value then break end
			p	=	p + 1
			text	=	self.value:sub(-p, -p) .. text
			if gc:getStringWidth(text) > (self.w - 8) then
				text	=	text:sub(2,-1)
				break
			end
		end
		--]]

		if self.disabled or self.value == "" then
				gc:setColorRGB(uCol(self.disabledcolor))
		end
		if self.value == ""  then
				text	= self.placeholder or ""
		end

		local strwidth = gc:getStringWidth(text)

		if strwidth<self.w-4 or not self.hasFocus then
				gc:drawString(text, x+2, y+1, "top")
		else
				gc:drawString(text, x-4+self.w-strwidth, y+1, "top")
		end

		if self.hasFocus and self.value ~= "" then
				gc:fillRect(self.x+(text==self.value and strwidth+2 or self.w-4), self.y, 1, self.h)
		end

		gc_clipRect(gc, "restore")
end

function sInput:charIn(char)
		if self.disabled or (self.number and not tonumber(self.value .. char)) then --or char~="," then
				return
		end
		--char = (char == ",") and "." or char
		self.value	=	self.value .. char
end

function sInput:clearKey()
    if self:deleteInvalid() then return 0 end
    self.value	=	""
end

function sInput:backspaceKey()
    if self:deleteInvalid() then return 0 end
		if not self.disabled then
				self.value	=	self.value:usub(1,-2)
		end
end

function sInput:deleteInvalid()
    local isInvalid = string.find(self.value, "Invalid input")
    if isInvalid then
        self.value = self.value:usub(1, -19)
        return true
    end
    return false
end

function sInput:enable()
		self.disabled	= false
end

function sInput:disable()
		self.disabled	= true
end




------------------------------------------------------------------
--                         Label Widget                         --
------------------------------------------------------------------

sLabel	=	class(Widget)

function sLabel:init(text, widget)
		self.widget	=	widget
		self.text	=	text
		self.ww		=	30

		self.hh		=	20
		self.lim	=	false
		self.color	=	{0,0,0}
		self.font	=	{"sansserif", "r", 10}
		self.p		=	"top"

end

function sLabel:paint(gc)
		gc:setFont(uCol(self.font))
		gc:setColorRGB(uCol(self.color))

		local text	=	""
		local ttext
		if self.lim then
				text, self.dw	= textLim(gc, self.text, self.w)
		else
				text = self.text
		end

		gc:drawString(text, self.x, self.y, self.p)
end

function sLabel:getFocus(n)
		if n then
				n	= n < 0 and -1 or (n > 0 and 1 or 0)
		end

		if self.widget and not n then
				self.widget:giveFocus()
		elseif n then
				self.parent:switchFocus(n)
		end
end


------------------------------------------------------------------
--                        Button Widget                         --
------------------------------------------------------------------

sButton	=	class(Widget)

function sButton:init(text, action)
    self.text	=	text
    self.action	=	action
    self.pushed = false

    self.dh	=	27
    self.dw	=	48

    self.bordercolor	=	{136,136,136}
    self.font	=	{"sansserif", "r", 10}
end

function sButton:paint(gc)
    gc:setFont(uCol(self.font))
    self.ww	=	gc:getStringWidth(self.text)+8
    self:size()

    if self.pushed and self.forcePushed then
        self.y = self.y + 2
    end

    gc:setColorRGB(248,252,248)
    gc:fillRect(self.x+2, self.y+2, self.w-4, self.h-4)
    gc:setColorRGB(0,0,0)

    gc:drawString(self.text, self.x+4, self.y+4, "top")

    if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:setPen("medium", "smooth")
    else
        gc:setColorRGB(uCol(self.bordercolor))
        gc:setPen("thin", "smooth")
    end
    gc:fillRect(self.x + 2, self.y, self.w-4, 2)
    gc:fillRect(self.x + 2, self.y+self.h-2, self.w-4, 2)
    gc:fillRect(self.x, self.y+2, 1, self.h-4)
    gc:fillRect(self.x+1, self.y+1, 1, self.h-2)
    gc:fillRect(self.x+self.w-1, self.y+2, 1, self.h-4)
    gc:fillRect(self.x+self.w-2, self.y+1, 1, self.h-2)

    if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        -- old way of indicating focus :
        --gc:drawRect(self.x-2, self.y-2, self.w+3, self.h+3)
        --gc:drawRect(self.x-3, self.y-3, self.w+5, self.h+5)
    end
end

function sButton:mouseMove(x,y)
    local isIn = (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h))
    self.pushed = self.forcePushed and isIn
    platform.window:invalidate()
end

function sButton:enterKey()
    if self.action then self.action() end
end

function sButton:mouseDown(x,y)
    if (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h)) then
        self.pushed = true
        self.forcePushed = true
    end
    platform.window:invalidate()
end

function sButton:mouseUp(x,y)
    self.pushed = false
    self.forcePushed = false
    if (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h)) then
        self:enterKey()
    end
    platform.window:invalidate()
end


------------------------------------------------------------------
--                      Scrollbar Widget                        --
------------------------------------------------------------------


scrollBar	= class(Widget)

scrollBar.upButton=image.new("\011\0\0\0\010\0\0\0\0\0\0\0\022\0\0\0\016\0\001\0001\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198\255\255\255\255\255\255\255\255\156\243\255\255\255\255\255\255\255\2551\1981\198\255\255\255\255\255\255\214\218\0\128\214\218\255\255\255\255\255\2551\1981\198\255\255\255\255\247\222B\136\0\128B\136\247\222\255\255\255\2551\1981\198\255\255\247\222B\136!\132\0\128!\132B\136\247\222\255\2551\1981\198\247\222B\136!\132B\136R\202B\136!\132B\136\247\2221\1981\198\132\144B\136B\136\247\222\255\255\247\222B\136B\136\132\1441\1981\198\156\243\132\144\247\222\255\255\255\255\255\255\247\222\132\144\189\2471\1981\198\255\255\222\251\255\255\255\255\255\255\255\255\255\255\222\251\255\2551\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198")
scrollBar.downButton=image.new("\011\0\0\0\010\0\0\0\0\0\0\0\022\0\0\0\016\0\001\0001\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198\255\255\222\251\255\255\255\255\255\255\255\255\255\255\222\251\255\2551\1981\198\156\243\132\144\247\222\255\255\255\255\255\255\247\222\132\144\189\2471\1981\198\132\144B\136B\136\247\222\255\255\247\222B\136B\136\132\1441\1981\198\247\222B\136!\132B\136R\202B\136!\132B\136\247\2221\1981\198\255\255\247\222B\136!\132\0\128!\132B\136\247\222\255\2551\1981\198\255\255\255\255\247\222B\136\0\128B\136\247\222\255\255\255\2551\1981\198\255\255\255\255\255\255\214\218\0\128\214\218\255\255\255\255\255\2551\1981\198\255\255\255\255\255\255\255\255\156\243\255\255\255\255\255\255\255\2551\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198")

function scrollBar:init(h, top, visible, total)
		self.color1	= {96, 100, 96}
		self.color2	= {184, 184, 184}

		self.hh	= h or 100
		self.ww = 14

		self.visible = visible or 10
		self.total   = total   or 15
		self.top     = top     or 4
end

function scrollBar:paint(gc)
		gc:setColorRGB(255,255,255)
		gc:fillRect(self.x+1, self.y+1, self.w-1, self.h-1)

		gc:drawImage(self.upButton  , self.x+2, self.y+2)
		gc:drawImage(self.downButton, self.x+2, self.y+self.h-11)
		gc:setColorRGB(uCol(self.color1))
		if self.h > 28 then
				gc:drawRect(self.x + 3, self.y + 14, 8, self.h - 28)
		end

		if self.visible<self.total then
				local step	= (self.h-26)/self.total
				gc:fillRect(self.x + 3, self.y + 14  + step*self.top, 9, step*self.visible)
				gc:setColorRGB(uCol(self.color2))
				gc:fillRect(self.x + 2 , self.y + 14 + step*self.top, 1, step*self.visible)
				gc:fillRect(self.x + 12, self.y + 14 + step*self.top, 1, step*self.visible)
		end
end

function scrollBar:update(top, visible, total)
		self.top      = top     or self.top
		self.visible  = visible or self.visible
		self.total    = total   or self.total
end

function scrollBar:action(top) end

function scrollBar:mouseUp(x, y)
		local upX	= self.x+2
		local upY	= self.y+2
		local downX	= self.x+2
		local downY	= self.y+self.h-11
		local butH	= 10
		local butW	= 11

		if x>=upX and x<upX+butW and y>=upY and y<upY+butH and self.top>0 then
				self.top	= self.top-1
				self:action(self.top)
		elseif x>=downX and x<downX+butW and y>=downY and y<downY+butH and self.top<self.total-self.visible then
				self.top	= self.top+1
				self:action(self.top)
		end
end

function scrollBar:getFocus(n)
		if n==-1 or n==1 then
				self.parent:switchFocus(n)
		end
end


------------------------------------------------------------------
--                         List Widget                          --
------------------------------------------------------------------

sList	= class(WWidget)

function sList:init()
		Widget.init(self)
		self.dw	= 150
		self.dh	= 153

		self.ih	= 18

		self.top	= 0
		self.sel	= 1

		self.font	= {"sansserif", "r", 10}
		self.colors	= {50,150,190}
		self.items	= {}
end

function sList:appended()
		self.scrollBar	= scrollBar("100", self.top, #self.items,#self.items)
		self:appendWidget(self.scrollBar, -1, 0)

		function self.scrollBar:action(top)
				self.parent.top	= top
		end
end


function sList:paint(gc)
		local x	= self.x
		local y	= self.y
		local w	= self.w
		local h	= self.h


		local ih	= self.ih
		local top	= self.top
		local sel	= self.sel

		local items	= self.items
		local visible_items	= math.floor(h/ih)
		gc:setColorRGB(255, 255, 255)
		gc:fillRect(x, y, w, h)
		gc:setColorRGB(0, 0, 0)
		gc:drawRect(x, y, w, h)
		gc_clipRect(gc, "set", x, y, w, h)
		gc:setFont(unpack(self.font))



		local label, item
		for i=1, math.min(#items-top, visible_items+1) do
				item	= items[i+top]
				label	= textLim(gc, item, w-(5 + 12 + 2 + 1))

				if i+top == sel then
						gc:setColorRGB(unpack(self.colors))
						gc:fillRect(x+1, y + i*ih-ih + 1, w-(12 + 2 + 2), ih)

						gc:setColorRGB(255, 255, 255)
				end

				gc:drawString(label, x+5, y + i*ih-ih , "top")
				gc:setColorRGB(0, 0, 0)
		end

		self.scrollBar:update(top, visible_items, #items)

		gc_clipRect(gc, "reset")
end

function sList:arrowKey(arrow)

		if arrow=="up" then
		    if self.sel>1 then
            self.sel	= self.sel - 1
            if self.top>=self.sel then
                self.top	= self.top - 1
            end
        else
            self.top = self.h/self.ih < #self.items and math.ceil(#self.items - self.h/self.ih) or 0
            self.sel = #self.items
        end
        self:change(self.sel, self.items[self.sel])
		end

		if arrow=="down" then
		    if self.sel<#self.items then
            self.sel	= self.sel + 1
            if self.sel>(self.h/self.ih)+self.top then
                self.top	= self.top + 1
            end
        else
            self.top = 0
            self.sel = 1
        end
        self:change(self.sel, self.items[self.sel])
		end
end


function sList:mouseUp(x, y)
		if x>=self.x and x<self.x+self.w-16 and y>=self.y and y<self.y+self.h then

			local sel	= math.floor((y-self.y)/self.ih) + 1 + self.top
			if sel==self.sel then
					self:enterKey()
					return
			end
			if self.items[sel] then
					self.sel=sel
					self:change(self.sel, self.items[self.sel])
			else
					return
			end

			if self.sel>(self.h/self.ih)+self.top then
					self.top	= self.top + 1
			end
			if self.top>=self.sel then
					self.top	= self.top - 1
			end

		end
		self.scrollBar:mouseUp(x, y)
end


function sList:enterKey()
		if self.items[self.sel] then
				self:action(self.sel, self.items[self.sel])
		end
end


function sList:change() end
function sList:action() end

function sList:reset()
		self.sel	= 1
		self.top	= 0
end

------------------------------------------------------------------
--                        Screen Widget                         --
------------------------------------------------------------------

sScreen	= class(WWidget)

function sScreen:init(w, h)
		Widget.init(self)
		self.ww	= w
		self.hh	= h
		self.oy	= 0
		self.ox	= 0
		self.noloop	= true
end

function sScreen:appended()
		self.oy	= 0
		self.ox	= 0
end

function sScreen:paint(gc)
		gc_clipRect(gc, "set", self.x, self.y, self.w, self.h)
		self.x	= self.x + self.ox
		self.y	= self.y + self.oy
end

function sScreen:postPaint(gc)
		gc_clipRect(gc, "reset")
end

function sScreen:setY(y)
		self.oy	= y or self.oy
end

function sScreen:setX(x)
		self.ox	= x or self.ox
end

function sScreen:showWidget()
		local w	= self:getWidget()
		if not w then print("bye") return end
		local y	= self.y - self.oy
		local wy = w.y - self.oy

		if w.y-2 < y then
				print("Moving up")
				self:setY(-(wy-y)+4)
		elseif w.y+w.h > y+self.h then
				print("moving down")
				self:setY(-(wy-(y+self.h)+w.h+2))
		end

		if self.focus == 1 then
			self:setY(0)
		end
end

function sScreen:getFocus(n)
		if n==-1 or n==1 then
				self:stealFocus()
				self:switchFocus(n, true)
		end
end

function sScreen:loop(n)
		self.parent:switchFocus(n)
		self:showWidget()
end

function sScreen:focusChange()
		self:showWidget()
end

function sScreen:loseFocus(n)
		if n and ((n >= 1 and self.focus+n<=#self.widgets) or (n <= -1 and self.focus+n>=1)) then
				self:switchFocus(n)
				return -1
		else
				self:stealFocus()
		end
end


-------------------------------------------------------------------------------
--									sDropdown							     --
-------------------------------------------------------------------------------

sDropdown	=	class(Widget)


function sDropdown:init(items)
		self.dh	= 21
		self.dw	= 75
		self.screen	= WScreen()
		self.sList	= sList()
		self.sList.items	= items or {}
		self.screen:appendWidget(self.sList,0,0)
		self.sList.action	= self.listAction
		self.sList.loseFocus	= self.screenEscape
		self.sList.change	= self.listChange
		self.screen.escapeKey	= self.screenEscape
		self.lak	= self.sList.arrowKey
		self.sList.arrowKey	= self.listArrowKey
		self.value	= items[1] or ""
		self.valuen	= #items>0 and 1 or 0
		self.rvalue	= items[1] or ""
		self.rvaluen=self.valuen

		self.sList.parentWidget = self
		self.screen.parentWidget = self
		--self.screen.focus=1
end

function sDropdown:screenpaint(gc)
		gc:setColorRGB(255,255,255)
		gc:fillRect(self.x, self.y, self.h, self.w)
		gc:setColorRGB(0,0,0)
		gc:drawRect(self.x, self.y, self.h, self.w)
end

function sDropdown:mouseDown()
		self:open()
end


sDropdown.img = image.new("\14\0\0\0\7\0\0\0\0\0\0\0\28\0\0\0\16\0\1\000{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239al{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239alalal{\239{\239\255\255\255\255\255\255\255\255\255\255\255\255{\239{\239alalalalal{\239{\239\255\255\255\255\255\255\255\255{\239{\239alalalalalalal{\239{\239\255\255\255\255{\239{\239alalalalalalalalal{\239{\239{\239{\239alalalalalalalalalalal{\239{\239alalalalalal")

function sDropdown:arrowKey(arrow)
		if arrow=="up" then
				self.parent:switchFocus(self.focusUp or -1)
		elseif arrow=="down" then
				self.parent:switchFocus(self.focusDown or 1)
		elseif arrow=="left" then
				self.parent:switchFocus(self.focusLeft or -1)
		elseif arrow == "right" then
				self:open()
		end
end

function sDropdown:listArrowKey(arrow)
		if arrow == "left" then
				self:loseFocus()
		else
				self.parentWidget.lak(self, arrow)
		end
end

function sDropdown:listChange(a, b)
		self.parentWidget.value  = b
		self.parentWidget.valuen = a
end

function sDropdown:open()
		self.screen.yy	= self.y+self.h
		self.screen.xx	= self.x-1
		self.screen.ww	= self.w + 13
		local h = 2+(18*#self.sList.items)

		local py	= self.parent.oy and self.parent.y-self.parent.oy or self.parent.y
		local ph	= self.parent.h

		self.screen.hh	= self.y+self.h+h>ph+py-10 and ph-py-self.y-self.h-10 or h
		if self.y+self.h+h>ph+py-10  and self.screen.hh<40 then
				self.screen.hh = h < self.y and h or self.y-5
				self.screen.yy = self.y-self.screen.hh
		end

		self.sList.ww = self.w + 13
		self.sList.hh = self.screen.hh-2
		self.sList.yy =self.sList.yy+1
		self.sList:giveFocus()

    self.screen:size()
		push_screen_direct(self.screen)
end

function sDropdown:listAction(a,b)
		self.parentWidget.value  = b
		self.parentWidget.valuen = a
		self.parentWidget.rvalue  = b
		self.parentWidget.rvaluen = a
		self.parentWidget:change(a, b)
		remove_screen()
end

function sDropdown:change() end

function sDropdown:screenEscape()
		self.parentWidget.sList.sel = self.parentWidget.rvaluen
		self.parentWidget.value	= self.parentWidget.rvalue
		if current_screen() == self.parentWidget.screen then
				remove_screen()
		end
end

function sDropdown:paint(gc)
		gc:setColorRGB(255, 255, 255)
		gc:fillRect(self.x, self.y, self.w-1, self.h-1)

		gc:setColorRGB(0,0,0)
		gc:drawRect(self.x, self.y, self.w-1, self.h-1)

		if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:drawRect(self.x-1, self.y-1, self.w+1, self.h+1)
        gc:setColorRGB(0, 0, 0)
    end

		gc:setColorRGB(192, 192, 192)
		gc:fillRect(self.x+self.w-21, self.y+1, 20, 19)
		gc:setColorRGB(224, 224, 224)
		gc:fillRect(self.x+self.w-22, self.y+1, 1, 19)

		gc:drawImage(self.img, self.x+self.w-18, self.y+9)

		gc:setColorRGB(0,0,0)
		local text = self.value
		if self.unitmode then
				text=text:gsub("([^%d]+)(%d)", numberToSub)
		end

		gc:drawString(textLim(gc, text, self.w-5-22), self.x+5, self.y, "top")
end
