--include "component.lua"
--include "node.lua"

--[[
Circuit Objects store Nodes and Components and solve for their respective unknowns
Attributes:
-- Nodes: A list of nodes in the circuit
-- Components: A list of components in the circuit
]]

Circuit = class()
	-- Circuit Constructor
function Circuit:init()
		self.nodes = {}
		self.components = {}
end
-- Solve the circuit given a node tied to ground
-- Sets voltage / current parameters for all components
---- as well as voltage parameters for every node
function Circuit:solve(ground)
		local eq = 'solve(system('
		for i, node in pairs(self.nodes) do
				if #node.voltageConnections > 0 then
						eq = eq .. node:getVoltageEq()
						eq = eq .. node:getSuperEq({[node.name] = true})
				else
						eq = eq .. node:getCurrentEq()
				end
		end
		local eval = eq .. ground.name .. ' = 0' .. '), {' .. self:getNodeList() .. '})'
		local ans = math.evalStr(eval)
		print(eval)
		print(ans)
		if ans then
				local solutions = split(ans, " and ")
				for i, solution in pairs(solutions) do
						local parts = split(solution, "=")
						for j, v in pairs(self.nodes) do
								if v.name == parts[1] then
										local number = string.gsub(math.evalStr(parts[2] .. "▶Decimal"), "%−", "-")
										parts = split(number, "")
										parts[2] = parts[2] or 0
										print(parts[2])
										v.voltage = tonumber(parts[1])*math.pow(10, tonumber(parts[2]))
										print(v.voltage)
								end
						end
				end
		end
		return ans
end

function split(s, delimiter)
    result = {};
    for match in (s..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match);
    end
    return result;
end

-- Returns a list of node names seperated by commas
function Circuit:getNodeList()
		local nodeList = ''
		for i, v in pairs(self.nodes) do
				nodeList = nodeList .. v.name .. ', '
		end
		if #nodeList > 0 then
		 		nodeList = string.sub(nodeList, 0, #nodeList-2)
		end
		return nodeList
end

Connection = class()
	-- Connection Constructor
	-- Used to store connected components and the side they are connected by
function Connection:init(component, sign)
		self.component = component
		self.sign = sign
end
