platform.apilevel = '2.0'
nspice_version= '0.1'
---**************************---
---**************************---
-- Start of new file
-- File: GUI/animation.lua
---**************************---
---**************************---


----------

local tstart = timer.start
function timer.start(ms)
    if not timer.isRunning then
        tstart(ms)
    end
    timer.isRunning = true
end

local tstop = timer.stop
function timer.stop()
    timer.isRunning = false
    tstop()
end


if platform.hw then
    timer.multiplier = platform.hw() < 4 and 3.2 or 1
else
    timer.multiplier = platform.isDeviceModeRendering() and 3.2 or 1
end

function on.timer()
    --current_screen():timer()
    local j = 1
    while j <= #timer.tasks do -- for each task
        if timer.tasks[j][2]() then -- delete it if has ended
            table.remove(timer.tasks, j)
            sj = j - 1
        end
        j = j + 1
    end
    if #timer.tasks > 0 then
        platform.window:invalidate()
    else
        --for _,screen in pairs(Screens) do
        --	screen:size()
        --end
        timer.stop()
    end
end

timer.tasks = {}

timer.addTask = function(object, task) timer.start(0.01) table.insert(timer.tasks, { object, task }) end

function timer.purgeTasks(object)
    local j = 1
    while j <= #timer.tasks do
        if timer.tasks[j][1] == object then
            table.remove(timer.tasks, j)
            j = j - 1
        end
        j = j + 1
    end
end


---------- Animable Object class
Object = class()
function Object:init(x, y, w, h, r)
    self.tasks = {}
    self.x = x
    self.y = y
    self.w = w
    self.h = h
    self.r = r
    self.visible = true
end

function Object:PushTask(task, t, ms, callback)
    table.insert(self.tasks, { task, t, ms, callback })
    timer.start(0.01)
    if #self.tasks == 1 then
        local ok = task(self, t, ms, callback)
        if not ok then table.remove(self.tasks, 1) end
    end
end

function Object:PopTask()
    table.remove(self.tasks, 1)
    if #self.tasks > 0 then
        local task, t, ms, callback = unpack(self.tasks[1])
        local ok = task(self, t, ms, callback)
        if not ok then table.remove(self.tasks, 1) end
    end
end

function Object:purgeTasks()
    for i = 1, #self.tasks do
        self.tasks[i] = nil
    end
    collectgarbage()
    timer.purgeTasks(self)
    self.tasks = {}
    return self
end

function Object:paint(gc)
    -- to override
end

speed = 1

function Object:__Animate(t, ms, callback)
    if not ms then ms = 50 end
    if ms < 0 then print("Error: Invalid time divisor (must be >= 0)") return end
    ms = ms / timer.multiplier
    if ms == 0 then ms = 1 end
    if not t or type(t) ~= "table" then print("Error: Target position is " .. type(t)) return end
    if not t.x then t.x = self.x end
    if not t.y then t.y = self.y end
    if not t.w then t.w = self.w end
    if not t.h then t.h = self.h end
    if not t.r then t.r = self.r else t.r = math.pi * t.r / 180 end
    local xinc = (t.x - self.x) / ms
    local xside = xinc >= 0 and 1 or -1
    local yinc = (t.y - self.y) / ms
    local yside = yinc >= 0 and 1 or -1
    local winc = (t.w - self.w) / ms
    local wside = winc >= 0 and 1 or -1
    local hinc = (t.h - self.h) / ms
    local hside = hinc >= 0 and 1 or -1
    local rinc = (t.r - self.r) / ms
    local rside = rinc >= 0 and 1 or -1
    timer.addTask(self, function()
        local b1, b2, b3, b4, b5 = false, false, false, false, false
        if (self.x + xinc * speed) * xside < t.x * xside then self.x = self.x + xinc * speed else b1 = true end
        if self.y * yside < t.y * yside then self.y = self.y + yinc * speed else b2 = true end
        if self.w * wside < t.w * wside then self.w = self.w + winc * speed else b3 = true end
        if self.h * hside < t.h * hside then self.h = self.h + hinc * speed else b4 = true end
        if self.r * rside < t.r * rside then self.r = self.r + rinc * speed else b5 = true end
        if self.w < 0 then self.w = 0 end
        if self.h < 0 then self.h = 0 end
        if b1 and b2 and b3 and b4 and b5 then
            self.x, self.y, self.w, self.h, self.r = t.x, t.y, t.w, t.h, t.r
            self:PopTask()
            if callback then callback(self) end
            return true
        end
        return false
    end)
    return true
end

function Object:__Delay(_, ms, callback)
    if not ms then ms = 50 end
    if ms < 0 then print("Error: Invalid time divisor (must be >= 0)") return end
    ms = ms / timer.multiplier
    if ms == 0 then ms = 1 end
    local t = 0
    timer.addTask(self, function()
        if t < ms then
            t = t + 1
            return false
        else
            self:PopTask()
            if callback then callback(self) end
            return true
        end
    end)
    return true
end

function Object:__setVisible(t, _, _)
    timer.addTask(self, function()
        self.visible = t
        self:PopTask()
        return true
    end)
    return true
end

function Object:Animate(t, ms, callback)
    self:PushTask(self.__Animate, t, ms, callback)
    return self
end

function Object:Delay(ms, callback)
    self:PushTask(self.__Delay, false, ms, callback)
    return self
end

function Object:setVisible(t)
    self:PushTask(self.__setVisible, t, 1, false)
    return self
end
---**************************---
---**************************---
-- Start of new file
-- File: GUI/globals.lua
---**************************---
---**************************---

------------------------------------------------------------------
--                  Overall Global Variables                    --
------------------------------------------------------------------


-- BetterLuaAPI : https://github.com/adriweb/BetterLuaAPI-for-TI-Nspire
--

a_acute = string.uchar(225)
a_circ  = string.uchar(226)
a_tilde = string.uchar(227)
a_diaer = string.uchar(228)
a_ring  = string.uchar(229)
e_acute = string.uchar(233)
e_grave = string.uchar(232)
o_acute = string.uchar(243)
o_circ  = string.uchar(244)
l_alpha = string.uchar(945)
l_beta = string.uchar(946)
l_omega = string.uchar(2126)
sup_plus = string.uchar(8314)
sup_minus = string.uchar(8315)
right_arrow = string.uchar(8594)


Color = {
	["black"] = {0, 0, 0},
	["red"] = {255, 0, 0},
	["green"] = {0, 255, 0},
	["blue "] = {0, 0, 255},
	["white"] = {255, 255, 255},
	["brown"] = {165,42,42},
	["cyan"] = {0,255,255},
	["darkblue"] = {0,0,139},
	["darkred"] = {139,0,0},
	["fuchsia"] = {255,0,255},
	["gold"] = {255,215,0},
	["gray"] = {127,127,127},
	["grey"] = {127,127,127},
	["lightblue"] = {173,216,230},
	["lightgreen"] = {144,238,144},
	["magenta"] = {255,0,255},
	["maroon"] = {128,0,0},
	["navyblue"] = {159,175,223},
	["orange"] = {255,165,0},
	["palegreen"] = {152,251,152},
	["pink"] = {255,192,203},
	["purple"] = {128,0,128},
	["royalblue"] = {65,105,225},
	["salmon"] = {250,128,114},
	["seagreen"] = {46,139,87},
	["silver"] = {192,192,192},
	["turquoise"] = {64,224,208},
	["violet"] = {238,130,238},
	["yellow"] = {255,255,0}
}
Color.mt = {__index = function () return {0,0,0} end}
setmetatable(Color,Color.mt)

function sign(v)
	if v > 0 then
			return 1
	elseif v < 0 then
			return -1
	end
	return 0
end

function copyTable(t)
  local t2 = {}
  for k,v in pairs(t) do
    t2[k] = v
  end
  return t2
end

function deepcopy(t) -- This function recursively copies a table's contents, and ensures that metatables are preserved. That is, it will correctly clone a pure Lua object.
	if type(t) ~= 'table' then return t end
	local mt = getmetatable(t)
	local res = {}
	for k,v in pairs(t) do
		if type(v) == 'table' then
		v = deepcopy(v)
		end
	res[k] = v
	end
	setmetatable(res,mt)
	return res
end -- from http://snippets.luacode.org/snippets/Deep_copy_of_a_Lua_Table_2

-- reverse table

function reverse(t)
		local i, j = 1, #t
		while j > i do
				t[i], t[j] = t[j], t[i]
				i = i + 1
				j = j - 1
		end
end

function utf8(nbr)
	return string.uchar(nbr)
end

function test(arg)
	return arg and 1 or 0
end

function screenRefresh()
	return platform.window:invalidate()
end

function pww()
	return platform.window:width()
end

function pwh()
	return platform.window:height()
end

function drawPoint(gc, x, y)
	gc:fillRect(x, y, 1, 1)
end

function drawCircle(gc, x, y, diameter)
	gc:drawArc(x - diameter/2, y - diameter/2, diameter,diameter,0,360)
end

function drawCenteredString(gc, str)
	gc:drawString(str, .5*(pww() - gc:getStringWidth(str)), .5*pwh(), "middle")
end

function drawXCenteredString(gc, str, y)
	gc:drawString(str, .5*(pww() - gc:getStringWidth(str)), y, "top")
end

function setColor(gc,theColor)
	if type(theColor) == "string" then
		theColor = string.lower(theColor)
		if type(Color[theColor]) == "table" then gc:setColorRGB(unpack(Color[theColor])) end
	elseif type(theColor) == "table" then
		gc:setColorRGB(unpack(theColor))
	end
end

function verticalBar(gc,x)
	gc:fillRect(gc,x,0,1,pwh())
end

function horizontalBar(gc,y)
	gc:fillRect(gc,0,y,pww(),1)
end

function nativeBar(gc, screen, y)
	gc:setColorRGB(128,128,128)
	gc:fillRect(screen.x+5, screen.y+y, screen.w-10, 2)
end

function drawSquare(gc,x,y,l)
	gc:drawPolyLine(gc,{(x-l/2),(y-l/2), (x+l/2),(y-l/2), (x+l/2),(y+l/2), (x-l/2),(y+l/2), (x-l/2),(y-l/2)})
end

function drawRoundRect(gc,x,y,wd,ht,rd)  -- wd = width, ht = height, rd = radius of the rounded corner
	x = x-wd/2  -- let the center of the square be the origin (x coord)
	y = y-ht/2 -- same for y coord
	if rd > ht/2 then rd = ht/2 end -- avoid drawing cool but unexpected shapes. This will draw a circle (max rd)
	gc:drawLine(x + rd, y, x + wd - (rd), y);
	gc:drawArc(x + wd - (rd*2), y + ht - (rd*2), rd*2, rd*2, 270, 90);
	gc:drawLine(x + wd, y + rd, x + wd, y + ht - (rd));
	gc:drawArc(x + wd - (rd*2), y, rd*2, rd*2,0,90);
	gc:drawLine(x + wd - (rd), y + ht, x + rd, y + ht);
	gc:drawArc(x, y, rd*2, rd*2, 90, 90);
	gc:drawLine(x, y + ht - (rd), x, y + rd);
	gc:drawArc(x, y + ht - (rd*2), rd*2, rd*2, 180, 90);
end

function fillRoundRect(gc,x,y,wd,ht,radius)  -- wd = width and ht = height -- renders badly when transparency (alpha) is not at maximum >< will re-code later
	if radius > ht/2 then radius = ht/2 end -- avoid drawing cool but unexpected shapes. This will draw a circle (max radius)
    gc:fillPolygon({(x-wd/2),(y-ht/2+radius), (x+wd/2),(y-ht/2+radius), (x+wd/2),(y+ht/2-radius), (x-wd/2),(y+ht/2-radius), (x-wd/2),(y-ht/2+radius)})
    gc:fillPolygon({(x-wd/2-radius+1),(y-ht/2), (x+wd/2-radius+1),(y-ht/2), (x+wd/2-radius+1),(y+ht/2), (x-wd/2+radius),(y+ht/2), (x-wd/2+radius),(y-ht/2)})
    x = x-wd/2  -- let the center of the square be the origin (x coord)
	y = y-ht/2 -- same
	gc:fillArc(x + wd - (radius*2), y + ht - (radius*2), radius*2, radius*2, 1, -91);
    gc:fillArc(x + wd - (radius*2), y, radius*2, radius*2,-2,91);
    gc:fillArc(x, y, radius*2, radius*2, 85, 95);
    gc:fillArc(x, y + ht - (radius*2), radius*2, radius*2, 180, 95);
end


-- Fullscreen 'Library'

doNotDisplayIcon = false

icon=image.new("\020\0\0\0\020\0\0\0\0\0\0\0\040\0\0\0\016\0\001\000wwwwwwwwwwwwww\223\251\222\251\189\251\188\251\188\251\221\255\221\255\254\255wwwwwwwwwwwwwwwwwwww\156\243\024\227\215\218\214\218\247\222\025\227Z\235\156\243wwwwwwwwwwwwwwwwwwwwww\024\227S\202s\206\181\214\214\218\248\2229\2279\231Z\235Z\235wwwwwwwwwwwwwwwwwwZ\235\207\185\016\194R\202s\206\148\210\214\218\214\218\214\2229\231Z\235:\231wwwwwwwwwwwwww\190\251\239\189\239\189\148\210\148\210\156\247\148\214\214\218\147\210\181\218{\239\025\227Z\235|\239wwwwwwwwwwww\149\214\239\189\239\189\239\189\206\185{\239\206\185R\202R\202\148\214{\239\247\2229\227Z\231wwwwwwwwww\189\255\016\194\239\189\239\189\239\189\206\185{\239\173\181\016\194\016\194s\210Z\235\214\218\247\222\025\227\189\247wwwwwwww8\243\016\194\239\189\239\189\240\189\206\189{\239\206\185\016\194\207\185s\2109\235\148\210\214\218\024\223{\239wwwwww\254\255\244\238\239\189\206\185\207\185\206\185\140\177z\239\008\161\008\161\198\152\016\194\214\218\173\181\017\194t\206:\231wwwwww\188\2556\247\016\194\206\185k\173)\165\231\156{\239\132\144\133\144c\140\239\193\148\210\008\161l\173\239\1899\231wwwwwwx\255\154\255\240\189\231\156\132\144C\136B\136k\173\0\128B\136!\132\165\148\231\156B\136\165\148K\173\156\243wwwwww6\255\154\255\024\227\198\152c\140\206\185\206\185\173\181\207\185k\173)\165\206\185\239\189J\169c\140\173\181\222\251wwwwww6\255x\255ww\140\177\0\128\148\210\016\194\173\181R\202\173\181\206\185R\202\239\189\231\156\164\152\213\218wwwwwwwwx\2556\255ww\222\251J\169\008\161c\140c\140c\140c\140c\140c\140\008\169O\230o\234\178\242z\251wwwwww\221\255\209\250wwwwww\239\189\132\144d\140B\136d\140\132\144B\136\202\213\012\234\012\230\012\230-\234\189\251wwwwww\242\2506\255wwwwww\156\243\149\210\016\194\240\1892\202\247\222\236\221\147\222r\2220\214\146\222\245\238wwwwww\188\255\141\250\243\250wwwwwwwwwwwwww\021\251\168\221\136\217\169\213\236\213O\222Y\243wwwwwwww\188\255\142\250m\250\244\250X\255y\2557\255\177\250)\246(\246K\242\168\229\134\229\134\229\178\238wwwwwwwwwwwwwwW\255\175\250k\250J\250K\250\141\250\242\250y\255ww\188\2557\251z\251wwwwwwwwwwwwwwwwwwww\222\255\222\255\222\255wwwwwwwwwwwwwwwwwwwwww")

local pw	= getmetatable(platform.window)
function pw:invalidateAll()
	if self.setFocus then
		self:setFocus(false)
		self:setFocus(true)
	end
end

function on.draw(gc)
	gc:setColorRGB(255, 255, 255)
	gc:fillRect(18, 5, 20, 20)
	gc:drawImage(icon, 18, 5)
end

if not platform.withGC then
    function platform.withGC(func, ...)
        local gc = platform.gc()
        gc:begin()
        func(..., gc)
        gc:finish()
    end
end
---**************************---
---**************************---
-- Start of new file
-- File: GUI/screen.lua
---**************************---
---**************************---



stdout	= print

function pprint(...)
	stdout(...)
	local out	= ""
	for _,v in ipairs({...}) do
		out	=	out .. (_==1 and "" or "    ") .. tostring(v)
	end
	var.store("print", out)
end


function Pr(n, d, s, ex)
	local nc	= tonumber(n)
	if nc and nc<math.abs(nc) then
		return s-ex-(type(n)== "number" and math.abs(n) or (.01*s*math.abs(nc)))
	else
		return (type(n)=="number" and n or (type(n)=="string" and .01*s*nc or d))
	end
end

-- Apply an extension on a class, and return our new frankenstein
function addExtension(oldclass, extension)
	local newclass	= class(oldclass)
	for key, data in pairs(extension) do
		newclass[key]	= data
	end
	return newclass
end

clipRectData	= {}

function gc_clipRect(gc, what, x, y, w, h)
	if what == "set" and clipRectData.current then
		clipRectData.old	= clipRectData.current

	elseif what == "subset" and clipRectData.current then
		clipRectData.old	= clipRectData.current
		x	= clipRectData.old.x<x and x or clipRectData.old.x
		y	= clipRectData.old.y<y and y or clipRectData.old.y
		h	= clipRectData.old.y+clipRectData.old.h > y+h and h or clipRectData.old.y+clipRectData.old.h-y
		w	= clipRectData.old.x+clipRectData.old.w > x+w and w or clipRectData.old.x+clipRectData.old.w-x
		what = "set"

	elseif what == "restore" and clipRectData.old then
		--gc:clipRect("reset")
		what = "set"
		x	= clipRectData.old.x
		y	= clipRectData.old.y
		h	= clipRectData.old.h
		w	= clipRectData.old.w
	elseif what == "restore" then
		what = "reset"
	end

	gc:clipRect(what, x, y, w, h)
	if x and y and w and h then clipRectData.current = {x=x,y=y,w=w,h=h} end
end

------------------------------------------------------------------
--                        Screen  Class                         --
------------------------------------------------------------------

Screen	=	class(Object)

Screens	=	{}

function scrollScreen(screen, d, callback)
  --  print("scrollScreen.  number of screens : ", #Screens)
    local dir = d or 1
    screen.x=dir*kXSize
    screen:Animate( {x=0}, 10, callback )
end

function insertScreen(screen, ...)
  --  print("insertScreen")
	screen:size()
    if current_screen() ~= DummyScreen then
        current_screen():screenLoseFocus()
        local coeff = pushFromBack and 1 or -1
	    current_screen():Animate( {x=coeff*kXSize}, 10)
    end
	table.insert(Screens, screen)

	platform.window:invalidate()
	current_screen():pushed(...)
end

function insertScreen_direct(screen, ...)
  --  print("insertScreen_direct")
	screen:size()
	table.insert(Screens, screen)
	platform.window:invalidate()
	current_screen():pushed(...)
end

function push_screen(screen, ...)
    --print("push_screen")
    local args = ...
    local theScreen = current_screen()
    pushFromBack = false
    insertScreen(screen, ...)
    scrollScreen(screen, 1, function() remove_screen_previous(theScreen) end)
end

function push_screen_back(screen, ...)
    --print("push_screen_back")
    local theScreen = current_screen()
    pushFromBack = true
    insertScreen(screen, ...)
    scrollScreen(screen, -1, function() remove_screen_previous(theScreen) end)
end

function push_screen_direct(screen, ...)
   -- print("push_screen_direct")
	table.insert(Screens, screen)
	platform.window:invalidate()
	current_screen():pushed(...)
end

function only_screen(screen, ...)
   -- print("only_screen")
    remove_screen(current_screen())
	Screens	=	{}
	push_screen(screen, ...)
	platform.window:invalidate()
end

function only_screen_back(screen, ...)
 --   print("only_screen_back")
    --Screens	=	{}
	push_screen_back(screen, ...)
	platform.window:invalidate()
end

function remove_screen_previous(...)
  --  print("remove_screen_previous")
	platform.window:invalidate()
	current_screen():removed(...)
	res=table.remove(Screens, #Screens-1)
	current_screen():screenGetFocus()
	return res
end

function remove_screen(...)
  --  print("remove_screen")
	platform.window:invalidate()
	current_screen():removed(...)
	res=table.remove(Screens)
	current_screen():screenGetFocus()
	return res
end

function current_screen()
	return Screens[#Screens] or DummyScreen
end

function Screen:init(xx,yy,ww,hh)

	self.yy	=	yy
	self.xx	=	xx
	self.hh	=	hh
	self.ww	=	ww

	self:ext()
	self:size(0)

	Object.init(self, self.x, self.y, self.w, self.h, 0)
end

function Screen:ext()
end

function Screen:size()
	local screenH	=	platform.window:height()
	local screenW	=	platform.window:width()

	if screenH	== 0 then screenH=212 end
	if screenW	== 0 then screenW=318 end

	self.x	=	math.floor(Pr(self.xx, 0, screenW)+.5)
	self.y	=	math.floor(Pr(self.yy, 0, screenH)+.5)
	self.w	=	math.floor(Pr(self.ww, screenW, screenW, 0)+.5)
	self.h	=	math.floor(Pr(self.hh, screenH, screenH, 0)+.5)
end


function Screen:pushed() end
function Screen:removed() end
function Screen:screenLoseFocus() end
function Screen:screenGetFocus() end

function Screen:draw(gc)
	--self:size()
	self:paint(gc)
end

function Screen:paint(gc) end

function Screen:invalidate()
	platform.window:invalidate(self.x ,self.y , self.w, self.h)
end

function Screen:arrowKey()	end
function Screen:enterKey()	end
function Screen:backspaceKey()	end
function Screen:clearKey() 	end
function Screen:escapeKey()	end
function Screen:tabKey()	end
function Screen:backtabKey()	end
function Screen:charIn(char)	end


function Screen:mouseDown()	end
function Screen:rightMouseDown()	end
function Screen:mouseUp()	end
function Screen:rightMouseUp()	end
function Screen:mouseMove()	end
function Screen:contextMenu()	end

function Screen:appended() end

function Screen:resize(w,h) end

function Screen:destroy()
	self	= nil
end

------------------------------------------------------------------
--                   WidgetManager Extension                    --
------------------------------------------------------------------

WidgetManager	= {}

function WidgetManager:ext()
	self.widgets	=	{}
	self.focus	=	0
end

function WidgetManager:resize(w,h)
    if self.x then  --already inited
        self:size()
    end
end

function WidgetManager:appendWidget(widget, xx, yy)
	widget.xx	=	xx
	widget.yy	=	yy
	widget.parent	=	self
	widget:size()

	table.insert(self.widgets, widget)
	widget.pid	=	#self.widgets

	widget:appended(self)
	return self
end

function WidgetManager:getWidget()
	return self.widgets[self.focus]
end

function WidgetManager:drawWidgets(gc)
	for _, widget in pairs(self.widgets) do
		widget:size()
		widget:draw(gc)

		gc:setColorRGB(0,0,0)
	end
end

function WidgetManager:postPaint(gc)
end

function WidgetManager:draw(gc)
	--self:size()
	self:paint(gc)
	self:drawWidgets(gc)
	self:postPaint(gc)
end


function WidgetManager:loop(n) end

function WidgetManager:stealFocus(n)
	local oldfocus=self.focus
	if oldfocus~=0 then
		local veto	= self:getWidget():loseFocus(n)
		if veto == -1 then
			return -1, oldfocus
		end
		self:getWidget().hasFocus	=	false
		self.focus	= 0
	end
	return 0, oldfocus
end

function WidgetManager:focusChange() end

function WidgetManager:switchFocus(n, b)
	if n~=0 and #self.widgets>0 then
		local veto, focus	= self:stealFocus(n)
		if veto == -1 then
			return -1
		end

		local looped
		self.focus	=	focus + n
		if self.focus>#self.widgets then
			self.focus	=	1
			looped	= true
		elseif self.focus<1 then
			self.focus	=	#self.widgets
			looped	= true
		end
		if looped and self.noloop and not b then
			self.focus	= focus
			self:loop(n)
		else
			self:getWidget().hasFocus	=	true
			self:getWidget():getFocus(n)
		end
	end
	self:focusChange()
end


function WidgetManager:arrowKey(arrow)
	if self.focus~=0 then
		self:getWidget():arrowKey(arrow)
	end
	self:invalidate()
end

function WidgetManager:enterKey()
    if self.focus~=0 then
        self:getWidget():enterKey()
    else
        if self.widgets and self.widgets[1] then   -- ugh, quite a bad hack for the mouseUp at (0,0) when cursor isn't shown (grrr TI) :/
            self.widgets[1]:enterKey()
        end
    end
    self:invalidate()
end

function WidgetManager:clearKey()
	if self.focus~=0 then
		self:getWidget():clearKey()
	end
	self:invalidate()
end

function WidgetManager:backspaceKey()
	if self.focus~=0 then
		self:getWidget():backspaceKey()
	end
	self:invalidate()
end

function WidgetManager:escapeKey()
	if self.focus~=0 then
		self:getWidget():escapeKey()
	end
	self:invalidate()
end

function WidgetManager:tabKey()
	self:switchFocus(1)
	self:invalidate()
end

function WidgetManager:backtabKey()
	self:switchFocus(-1)
	self:invalidate()
end

function WidgetManager:charIn(char)
	if self.focus~=0 then
		self:getWidget():charIn(char)
	end
	self:invalidate()
end

function WidgetManager:getWidgetIn(x, y)
	for n, widget in pairs(self.widgets) do
		local wox	= widget.ox or 0
		local woy	= widget.oy or 0
		if x>=widget.x-wox and y>=widget.y-wox and x<widget.x+widget.w-wox and y<widget.y+widget.h-woy then
			return n, widget
		end
	end
end

function WidgetManager:mouseDown(x, y)
		local n, widget	=	self:getWidgetIn(x, y)
		if n then
			if self.focus~=0 and self.focus~=n then self:getWidget().hasFocus = false self:getWidget():loseFocus()  end
			self.focus	=	n

			widget.hasFocus	=	true
			widget:getFocus()

			widget:mouseDown(x, y)
			self:focusChange()
		else
			if self.focus~=0 then self:getWidget().hasFocus = false self:getWidget():loseFocus() end
			self.focus	=	0
		end
end

function WidgetManager:mouseUp(x, y)
    if self.focus~=0 then
        --self:getWidget():mouseUp(x, y)
    end
    for _, widget in pairs(self.widgets) do
        widget:mouseUp(x,y) -- well, mouseUp is a release of a button, so everything previously "clicked" should be released, for every widget, even if the mouse has moved (and thus changed widget)
        -- eventually, a better way for this would be to keep track of the last widget active and do it to this one only...
    end
    self:invalidate()
end

function WidgetManager:rightMouseUp(x, y)
    for _, widget in pairs(self.widgets) do
        widget:rightMouseUp(x,y)
    end
    self:invalidate()
end

function WidgetManager:rightMouseDown(x, y)
		local n, widget	=	self:getWidgetIn(x, y)
		if n then
			if self.focus~=0 and self.focus~=n then self:getWidget().hasFocus = false self:getWidget():loseFocus()  end
			self.focus	=	n

			widget.hasFocus	=	true
			widget:getFocus()

			widget:rightMouseDown(x, y)
			self:focusChange()
		else
			if self.focus~=0 then self:getWidget().hasFocus = false self:getWidget():loseFocus() end
			self.focus	=	0
		end
end

function WidgetManager:mouseMove(x, y)
	if self.focus~=0 then
		self:getWidget():mouseMove(x, y)
	end
end



--------------------------
-- Our new frankenstein --
--------------------------

WScreen	= addExtension(Screen, WidgetManager)

--Dialog screen

Dialog	=	class(WScreen)

function Dialog:init(title,xx,yy,ww,hh)

	self.yy	=	yy
	self.xx	=	xx
	self.hh	=	hh
	self.ww	=	ww
	self.title	=	title
	self:size()

	self.widgets	=	{}
	self.focus	=	0

end

function drawDialog(gc, title, x, y, width, height)
	gc:setFont("sansserif", "r", 10)
	gc:setColorRGB(224, 224, 224)

	gc:fillRect(x, y, width, height)

	for i=1, 14, 2 do
			gc:setColorRGB(32+i*3, 32+i*4, 32+i*3)
			gc:fillRect(x, y + i, width, 2)
	end

	gc:setColorRGB(32+16*3, 32+16*4, 32+16*3)
	gc:fillRect(x, y+15, width, 10)

	gc:setColorRGB(128,128,128)
	gc:drawRect(x, y, width, height)
	gc:drawRect(x-1, y-1, width+2, height+2)

	gc:setColorRGB(96, 100, 96)
	gc:fillRect(x+width+1, y, 1, height+2)
	gc:fillRect(x, y+height+2, width+3, 1)

	gc:setColorRGB(104, 108, 104)
	gc:fillRect(x+width+2, y+1, 1, height+2)
	gc:fillRect(x+1, y+height+3, width+3, 1)
	gc:fillRect(x+width+3, y+2, 1, height+2)
	gc:fillRect(x+2, y+height+4, width+2, 1)

	gc:setColorRGB(255, 255, 255)
	gc:drawString(title, x+4, y+2, "top")
	gc:setColorRGB(0, 0, 0)
end

function Dialog:paint(gc)
	drawDialog(gc, self.title, self.x, self.y, self.w, self.h)
	self:postPaint(gc)
end

function Dialog:postPaint() end



---
-- The dummy screen
---

DummyScreen	= Screen()


------------------------------------------------------------------
--                   Bindings to the on events                  --
------------------------------------------------------------------


function on.paint(gc)
    -- allWentWell, generalErrMsg = pcall(onpaint, gc)
    -- if not allWentWell and errorHandler then
    --     errorHandler.display = true
    --     errorHandler.errorMessage = generalErrMsg
    -- end
		onpaint(gc)
    if platform.hw and platform.hw() < 4 and not doNotDisplayIcon then
    	platform.withGC(on.draw)
    end
end

function onpaint(gc)
	for _, screen in pairs(Screens) do
		screen:draw(gc)
	end
	-- if errorHandler.display then
	--     errorPopup(gc)
	-- end
end

kXRatio = 1
kYRatio = 1

kXSize, kYSize = 1, 1

function on.resize(w, h)
	-- Global Ratio Constants for On-Calc (shouldn't be used often though...)
	kXRatio = w/320
	kYRatio = h/212

	kXSize, kYSize = w, h

	for _,screen in pairs(Screens) do
		screen:resize(w,h)
	end
end

function on.arrowKey(arrw)	current_screen():arrowKey(arrw)  end
function on.enterKey()		current_screen():enterKey()		 end
function on.escapeKey()		current_screen():escapeKey()	 end
function on.tabKey()		current_screen():tabKey()		 end
function on.backtabKey()	current_screen():backtabKey()	 end
function on.charIn(ch)		current_screen():charIn(ch)		 end
function on.backspaceKey()	current_screen():backspaceKey()  end
function on.contextMenu()	current_screen():contextMenu()   end
function on.mouseDown(x,y)	current_screen():mouseDown(x,y)	 end
function on.mouseUp(x,y)	if (x == 0 and y == 0) then current_screen():enterKey() else current_screen():mouseUp(x,y) end	 end
function on.rightMouseDown(x,y)	current_screen():rightMouseDown(x,y)	 end
function on.rightMouseUp(x,y)	current_screen():rightMouseUp(x,y)	 end
function on.mouseMove(x,y)	current_screen():mouseMove(x,y)  end
function on.clearKey()    	current_screen():clearKey()      end
---**************************---
---**************************---
-- Start of new file
-- File: GUI/widgets.lua
---**************************---
---**************************---


function uCol(col)
		return col[1] or 0, col[2] or 0, col[3] or 0
end

function textLim(gc, text, max)
		local ttext, out = "",""
		local width	= gc:getStringWidth(text)
		if width<max then
				return text, width
		else
				for i=1, #text do
						ttext	= text:usub(1, i)
						if gc:getStringWidth(ttext .. "..")>max then
								break
						end
						out = ttext
				end
				return out .. "..", gc:getStringWidth(out .. "..")
		end
end


------------------------------------------------------------------
--                        Widget  Class                         --
------------------------------------------------------------------

Widget	=	class(Screen)

function Widget:init()
		self.dw	=	10
		self.dh	=	10

		self:ext()
end

function Widget:setSize(w, h)
		self.ww	= w or self.ww
		self.hh	= h or self.hh
end

function Widget:setPos(x, y)
		self.xx	= x or self.xx
		self.yy	= y or self.yy
end

function Widget:size(n)
		if n then return end
		self.w	=	math.floor(Pr(self.ww, self.dw, self.parent.w, 0)+.5)
		self.h	=	math.floor(Pr(self.hh, self.dh, self.parent.h, 0)+.5)

		self.rx	=	math.floor(Pr(self.xx, 0, self.parent.w, self.w)+.5)
		self.ry	=	math.floor(Pr(self.yy, 0, self.parent.h, self.h)+.5)
		self.x	=	self.parent.x + self.rx
		self.y	=	self.parent.y + self.ry
end

function Widget:giveFocus()
		if self.parent.focus~=0 then
				local veto	= self.parent:stealFocus()
				if veto == -1 then
						return -1
				end
		end

		self.hasFocus	=	true
		self.parent.focus	=	self.pid
		self:getFocus()
end

function Widget:getFocus() end
function Widget:loseFocus() end
function Widget:clearKey() 	end

function Widget:enterKey()
		self.parent:switchFocus(1)
end
function Widget:arrowKey(arrow)
		if arrow=="up" then
				self.parent:switchFocus(self.focusUp or -1)
		elseif arrow=="down"  then
				self.parent:switchFocus(self.focusDown or 1)
		elseif arrow=="left" then
				self.parent:switchFocus(self.focusLeft or -1)
		elseif arrow=="right"  then
				self.parent:switchFocus(self.focusRight or 1)
		end
end


WWidget	= addExtension(Widget, WidgetManager)


------------------------------------------------------------------
--                        Sample Widget                         --
------------------------------------------------------------------

-- First, create a new class based on Widget
box	=	class(Widget)

-- Init. You should define self.dh and self.dw, in case the user doesn't supply correct width/height values.
-- self.ww and self.hh can be a number or a string. If it's a number, the width will be that amount of pixels.
-- If it's a string, it will be interpreted as % of the parent screen size.
-- These values will be used to calculate self.w and self.h (don't write to this, it will overwritten everytime the widget get's painted)
-- self.xx and self.yy will be set when appending the widget to a screen. This value support the same % method (in a string)
-- They will be used to calculate self.x and self.h
function box:init(ww,hh,t)
		self.dh	= 10
		self.dw	= 10
		self.ww	= ww
		self.hh	= hh
		self.t	= t
end

-- Paint. Here you can paint your widget stuff
-- Variable you can use:
-- self.x, self.y	: numbers, x and y coordinates of the widget relative to screen. So it's the actual pixel position on the screen.
-- self.w, self.h	: numbers, w and h of widget
-- many others

function box:paint(gc)
		gc:setColorRGB(0,0,0)

		-- Do I have focus?
		if self.hasFocus then
			-- Yes, draw a filled black square
				gc:fillRect(self.x, self.y, self.w, self.h)
		else
			-- No, draw only the outline
				gc:drawRect(self.x, self.y, self.w, self.h)
		end

		gc:setColorRGB(128,128,128)
		if self.t then
				gc:drawString(self.t,self.x+2,self.y+2,"top")
		end
end


------------------------------------------------------------------
--                         Input Widget                         --
------------------------------------------------------------------


sInput	=	class(Widget)

function sInput:init()
		self.dw	=	100
		self.dh	=	20

		self.value	=	""
		self.bgcolor	=	{255,255,255}
		self.disabledcolor	= {128,128,128}
		self.font	=	{"sansserif", "r", 10}
		self.disabled	= false
end

function sInput:paint(gc)
		self.gc	=	gc
		local x	=	self.x
		local y = 	self.y

		gc:setPen("thin")
		gc:setFont(uCol(self.font))
		gc:setColorRGB(uCol(self.bgcolor))
		gc:fillRect(x, y, self.w, self.h)

		gc:setColorRGB(0,0,0)
		gc:drawRect(x, y, self.w, self.h)

		if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:drawRect(x-1, y-1, self.w+2, self.h+2)
        gc:setColorRGB(0, 0, 0)
    end

		local text	=	self.value
		local	p	=	0


		gc_clipRect(gc, "subset", x, y, self.w, self.h)

		--[[
		while true do
			if p==#self.value then break end
			p	=	p + 1
			text	=	self.value:sub(-p, -p) .. text
			if gc:getStringWidth(text) > (self.w - 8) then
				text	=	text:sub(2,-1)
				break
			end
		end
		--]]

		if self.disabled or self.value == "" then
				gc:setColorRGB(uCol(self.disabledcolor))
		end
		if self.value == ""  then
				text	= self.placeholder or ""
		end

		local strwidth = gc:getStringWidth(text)

		if strwidth<self.w-4 or not self.hasFocus then
				gc:drawString(text, x+2, y+1, "top")
		else
				gc:drawString(text, x-4+self.w-strwidth, y+1, "top")
		end

		if self.hasFocus and self.value ~= "" then
				gc:fillRect(self.x+(text==self.value and strwidth+2 or self.w-4), self.y, 1, self.h)
		end

		gc_clipRect(gc, "restore")
end

function sInput:charIn(char)
		if self.disabled or (self.number and not tonumber(self.value .. char)) then --or char~="," then
				return
		end
		--char = (char == ",") and "." or char
		self.value	=	self.value .. char
end

function sInput:clearKey()
    if self:deleteInvalid() then return 0 end
    self.value	=	""
end

function sInput:backspaceKey()
    if self:deleteInvalid() then return 0 end
		if not self.disabled then
				self.value	=	self.value:usub(1,-2)
		end
end

function sInput:deleteInvalid()
    local isInvalid = string.find(self.value, "Invalid input")
    if isInvalid then
        self.value = self.value:usub(1, -19)
        return true
    end
    return false
end

function sInput:enable()
		self.disabled	= false
end

function sInput:disable()
		self.disabled	= true
end




------------------------------------------------------------------
--                         Label Widget                         --
------------------------------------------------------------------

sLabel	=	class(Widget)

function sLabel:init(text, widget)
		self.widget	=	widget
		self.text	=	text
		self.ww		=	30

		self.hh		=	20
		self.lim	=	false
		self.color	=	{0,0,0}
		self.font	=	{"sansserif", "r", 10}
		self.p		=	"top"

end

function sLabel:paint(gc)
		gc:setFont(uCol(self.font))
		gc:setColorRGB(uCol(self.color))

		local text	=	""
		local ttext
		if self.lim then
				text, self.dw	= textLim(gc, self.text, self.w)
		else
				text = self.text
		end

		gc:drawString(text, self.x, self.y, self.p)
end

function sLabel:getFocus(n)
		if n then
				n	= n < 0 and -1 or (n > 0 and 1 or 0)
		end

		if self.widget and not n then
				self.widget:giveFocus()
		elseif n then
				self.parent:switchFocus(n)
		end
end


------------------------------------------------------------------
--                        Button Widget                         --
------------------------------------------------------------------

sButton	=	class(Widget)

function sButton:init(text, action)
    self.text	=	text
    self.action	=	action
    self.pushed = false

    self.dh	=	27
    self.dw	=	48

    self.bordercolor	=	{136,136,136}
    self.font	=	{"sansserif", "r", 10}
end

function sButton:paint(gc)
    gc:setFont(uCol(self.font))
    self.ww	=	gc:getStringWidth(self.text)+8
    self:size()

    if self.pushed and self.forcePushed then
        self.y = self.y + 2
    end

    gc:setColorRGB(248,252,248)
    gc:fillRect(self.x+2, self.y+2, self.w-4, self.h-4)
    gc:setColorRGB(0,0,0)

    gc:drawString(self.text, self.x+4, self.y+4, "top")

    if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:setPen("medium", "smooth")
    else
        gc:setColorRGB(uCol(self.bordercolor))
        gc:setPen("thin", "smooth")
    end
    gc:fillRect(self.x + 2, self.y, self.w-4, 2)
    gc:fillRect(self.x + 2, self.y+self.h-2, self.w-4, 2)
    gc:fillRect(self.x, self.y+2, 1, self.h-4)
    gc:fillRect(self.x+1, self.y+1, 1, self.h-2)
    gc:fillRect(self.x+self.w-1, self.y+2, 1, self.h-4)
    gc:fillRect(self.x+self.w-2, self.y+1, 1, self.h-2)

    if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        -- old way of indicating focus :
        --gc:drawRect(self.x-2, self.y-2, self.w+3, self.h+3)
        --gc:drawRect(self.x-3, self.y-3, self.w+5, self.h+5)
    end
end

function sButton:mouseMove(x,y)
    local isIn = (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h))
    self.pushed = self.forcePushed and isIn
    platform.window:invalidate()
end

function sButton:enterKey()
    if self.action then self.action() end
end

function sButton:mouseDown(x,y)
    if (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h)) then
        self.pushed = true
        self.forcePushed = true
    end
    platform.window:invalidate()
end

function sButton:mouseUp(x,y)
    self.pushed = false
    self.forcePushed = false
    if (x>self.x and x<(self.x+self.w) and y>self.y and y<(self.y+self.h)) then
        self:enterKey()
    end
    platform.window:invalidate()
end


------------------------------------------------------------------
--                      Scrollbar Widget                        --
------------------------------------------------------------------


scrollBar	= class(Widget)

scrollBar.upButton=image.new("\011\0\0\0\010\0\0\0\0\0\0\0\022\0\0\0\016\0\001\0001\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198\255\255\255\255\255\255\255\255\156\243\255\255\255\255\255\255\255\2551\1981\198\255\255\255\255\255\255\214\218\0\128\214\218\255\255\255\255\255\2551\1981\198\255\255\255\255\247\222B\136\0\128B\136\247\222\255\255\255\2551\1981\198\255\255\247\222B\136!\132\0\128!\132B\136\247\222\255\2551\1981\198\247\222B\136!\132B\136R\202B\136!\132B\136\247\2221\1981\198\132\144B\136B\136\247\222\255\255\247\222B\136B\136\132\1441\1981\198\156\243\132\144\247\222\255\255\255\255\255\255\247\222\132\144\189\2471\1981\198\255\255\222\251\255\255\255\255\255\255\255\255\255\255\222\251\255\2551\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198")
scrollBar.downButton=image.new("\011\0\0\0\010\0\0\0\0\0\0\0\022\0\0\0\016\0\001\0001\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198\255\255\222\251\255\255\255\255\255\255\255\255\255\255\222\251\255\2551\1981\198\156\243\132\144\247\222\255\255\255\255\255\255\247\222\132\144\189\2471\1981\198\132\144B\136B\136\247\222\255\255\247\222B\136B\136\132\1441\1981\198\247\222B\136!\132B\136R\202B\136!\132B\136\247\2221\1981\198\255\255\247\222B\136!\132\0\128!\132B\136\247\222\255\2551\1981\198\255\255\255\255\247\222B\136\0\128B\136\247\222\255\255\255\2551\1981\198\255\255\255\255\255\255\214\218\0\128\214\218\255\255\255\255\255\2551\1981\198\255\255\255\255\255\255\255\255\156\243\255\255\255\255\255\255\255\2551\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\1981\198")

function scrollBar:init(h, top, visible, total)
		self.color1	= {96, 100, 96}
		self.color2	= {184, 184, 184}

		self.hh	= h or 100
		self.ww = 14

		self.visible = visible or 10
		self.total   = total   or 15
		self.top     = top     or 4
end

function scrollBar:paint(gc)
		gc:setColorRGB(255,255,255)
		gc:fillRect(self.x+1, self.y+1, self.w-1, self.h-1)

		gc:drawImage(self.upButton  , self.x+2, self.y+2)
		gc:drawImage(self.downButton, self.x+2, self.y+self.h-11)
		gc:setColorRGB(uCol(self.color1))
		if self.h > 28 then
				gc:drawRect(self.x + 3, self.y + 14, 8, self.h - 28)
		end

		if self.visible<self.total then
				local step	= (self.h-26)/self.total
				gc:fillRect(self.x + 3, self.y + 14  + step*self.top, 9, step*self.visible)
				gc:setColorRGB(uCol(self.color2))
				gc:fillRect(self.x + 2 , self.y + 14 + step*self.top, 1, step*self.visible)
				gc:fillRect(self.x + 12, self.y + 14 + step*self.top, 1, step*self.visible)
		end
end

function scrollBar:update(top, visible, total)
		self.top      = top     or self.top
		self.visible  = visible or self.visible
		self.total    = total   or self.total
end

function scrollBar:action(top) end

function scrollBar:mouseUp(x, y)
		local upX	= self.x+2
		local upY	= self.y+2
		local downX	= self.x+2
		local downY	= self.y+self.h-11
		local butH	= 10
		local butW	= 11

		if x>=upX and x<upX+butW and y>=upY and y<upY+butH and self.top>0 then
				self.top	= self.top-1
				self:action(self.top)
		elseif x>=downX and x<downX+butW and y>=downY and y<downY+butH and self.top<self.total-self.visible then
				self.top	= self.top+1
				self:action(self.top)
		end
end

function scrollBar:getFocus(n)
		if n==-1 or n==1 then
				self.parent:switchFocus(n)
		end
end


------------------------------------------------------------------
--                         List Widget                          --
------------------------------------------------------------------

sList	= class(WWidget)

function sList:init()
		Widget.init(self)
		self.dw	= 150
		self.dh	= 153

		self.ih	= 18

		self.top	= 0
		self.sel	= 1

		self.font	= {"sansserif", "r", 10}
		self.colors	= {50,150,190}
		self.items	= {}
end

function sList:appended()
		self.scrollBar	= scrollBar("100", self.top, #self.items,#self.items)
		self:appendWidget(self.scrollBar, -1, 0)

		function self.scrollBar:action(top)
				self.parent.top	= top
		end
end


function sList:paint(gc)
		local x	= self.x
		local y	= self.y
		local w	= self.w
		local h	= self.h


		local ih	= self.ih
		local top	= self.top
		local sel	= self.sel

		local items	= self.items
		local visible_items	= math.floor(h/ih)
		gc:setColorRGB(255, 255, 255)
		gc:fillRect(x, y, w, h)
		gc:setColorRGB(0, 0, 0)
		gc:drawRect(x, y, w, h)
		gc_clipRect(gc, "set", x, y, w, h)
		gc:setFont(unpack(self.font))



		local label, item
		for i=1, math.min(#items-top, visible_items+1) do
				item	= items[i+top]
				label	= textLim(gc, item, w-(5 + 12 + 2 + 1))

				if i+top == sel then
						gc:setColorRGB(unpack(self.colors))
						gc:fillRect(x+1, y + i*ih-ih + 1, w-(12 + 2 + 2), ih)

						gc:setColorRGB(255, 255, 255)
				end

				gc:drawString(label, x+5, y + i*ih-ih , "top")
				gc:setColorRGB(0, 0, 0)
		end

		self.scrollBar:update(top, visible_items, #items)

		gc_clipRect(gc, "reset")
end

function sList:arrowKey(arrow)

		if arrow=="up" then
		    if self.sel>1 then
            self.sel	= self.sel - 1
            if self.top>=self.sel then
                self.top	= self.top - 1
            end
        else
            self.top = self.h/self.ih < #self.items and math.ceil(#self.items - self.h/self.ih) or 0
            self.sel = #self.items
        end
        self:change(self.sel, self.items[self.sel])
		end

		if arrow=="down" then
		    if self.sel<#self.items then
            self.sel	= self.sel + 1
            if self.sel>(self.h/self.ih)+self.top then
                self.top	= self.top + 1
            end
        else
            self.top = 0
            self.sel = 1
        end
        self:change(self.sel, self.items[self.sel])
		end
end


function sList:mouseUp(x, y)
		if x>=self.x and x<self.x+self.w-16 and y>=self.y and y<self.y+self.h then

			local sel	= math.floor((y-self.y)/self.ih) + 1 + self.top
			if sel==self.sel then
					self:enterKey()
					return
			end
			if self.items[sel] then
					self.sel=sel
					self:change(self.sel, self.items[self.sel])
			else
					return
			end

			if self.sel>(self.h/self.ih)+self.top then
					self.top	= self.top + 1
			end
			if self.top>=self.sel then
					self.top	= self.top - 1
			end

		end
		self.scrollBar:mouseUp(x, y)
end


function sList:enterKey()
		if self.items[self.sel] then
				self:action(self.sel, self.items[self.sel])
		end
end


function sList:change() end
function sList:action() end

function sList:reset()
		self.sel	= 1
		self.top	= 0
end

------------------------------------------------------------------
--                        Screen Widget                         --
------------------------------------------------------------------

sScreen	= class(WWidget)

function sScreen:init(w, h)
		Widget.init(self)
		self.ww	= w
		self.hh	= h
		self.oy	= 0
		self.ox	= 0
		self.noloop	= true
end

function sScreen:appended()
		self.oy	= 0
		self.ox	= 0
end

function sScreen:paint(gc)
		gc_clipRect(gc, "set", self.x, self.y, self.w, self.h)
		self.x	= self.x + self.ox
		self.y	= self.y + self.oy
end

function sScreen:postPaint(gc)
		gc_clipRect(gc, "reset")
end

function sScreen:setY(y)
		self.oy	= y or self.oy
end

function sScreen:setX(x)
		self.ox	= x or self.ox
end

function sScreen:showWidget()
		local w	= self:getWidget()
		if not w then print("bye") return end
		local y	= self.y - self.oy
		local wy = w.y - self.oy

		if w.y-2 < y then
				print("Moving up")
				self:setY(-(wy-y)+4)
		elseif w.y+w.h > y+self.h then
				print("moving down")
				self:setY(-(wy-(y+self.h)+w.h+2))
		end

		if self.focus == 1 then
			self:setY(0)
		end
end

function sScreen:getFocus(n)
		if n==-1 or n==1 then
				self:stealFocus()
				self:switchFocus(n, true)
		end
end

function sScreen:loop(n)
		self.parent:switchFocus(n)
		self:showWidget()
end

function sScreen:focusChange()
		self:showWidget()
end

function sScreen:loseFocus(n)
		if n and ((n >= 1 and self.focus+n<=#self.widgets) or (n <= -1 and self.focus+n>=1)) then
				self:switchFocus(n)
				return -1
		else
				self:stealFocus()
		end
end


-------------------------------------------------------------------------------
--									sDropdown							     --
-------------------------------------------------------------------------------

sDropdown	=	class(Widget)


function sDropdown:init(items)
		self.dh	= 21
		self.dw	= 75
		self.screen	= WScreen()
		self.sList	= sList()
		self.sList.items	= items or {}
		self.screen:appendWidget(self.sList,0,0)
		self.sList.action	= self.listAction
		self.sList.loseFocus	= self.screenEscape
		self.sList.change	= self.listChange
		self.screen.escapeKey	= self.screenEscape
		self.lak	= self.sList.arrowKey
		self.sList.arrowKey	= self.listArrowKey
		self.value	= items[1] or ""
		self.valuen	= #items>0 and 1 or 0
		self.rvalue	= items[1] or ""
		self.rvaluen=self.valuen

		self.sList.parentWidget = self
		self.screen.parentWidget = self
		--self.screen.focus=1
end

function sDropdown:screenpaint(gc)
		gc:setColorRGB(255,255,255)
		gc:fillRect(self.x, self.y, self.h, self.w)
		gc:setColorRGB(0,0,0)
		gc:drawRect(self.x, self.y, self.h, self.w)
end

function sDropdown:mouseDown()
		self:open()
end


sDropdown.img = image.new("\14\0\0\0\7\0\0\0\0\0\0\0\28\0\0\0\16\0\1\000{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239al{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239{\239alalal{\239{\239\255\255\255\255\255\255\255\255\255\255\255\255{\239{\239alalalalal{\239{\239\255\255\255\255\255\255\255\255{\239{\239alalalalalalal{\239{\239\255\255\255\255{\239{\239alalalalalalalalal{\239{\239{\239{\239alalalalalalalalalalal{\239{\239alalalalalal")

function sDropdown:arrowKey(arrow)
		if arrow=="up" then
				self.parent:switchFocus(self.focusUp or -1)
		elseif arrow=="down" then
				self.parent:switchFocus(self.focusDown or 1)
		elseif arrow=="left" then
				self.parent:switchFocus(self.focusLeft or -1)
		elseif arrow == "right" then
				self:open()
		end
end

function sDropdown:listArrowKey(arrow)
		if arrow == "left" then
				self:loseFocus()
		else
				self.parentWidget.lak(self, arrow)
		end
end

function sDropdown:listChange(a, b)
		self.parentWidget.value  = b
		self.parentWidget.valuen = a
end

function sDropdown:open()
		self.screen.yy	= self.y+self.h
		self.screen.xx	= self.x-1
		self.screen.ww	= self.w + 13
		local h = 2+(18*#self.sList.items)

		local py	= self.parent.oy and self.parent.y-self.parent.oy or self.parent.y
		local ph	= self.parent.h

		self.screen.hh	= self.y+self.h+h>ph+py-10 and ph-py-self.y-self.h-10 or h
		if self.y+self.h+h>ph+py-10  and self.screen.hh<40 then
				self.screen.hh = h < self.y and h or self.y-5
				self.screen.yy = self.y-self.screen.hh
		end

		self.sList.ww = self.w + 13
		self.sList.hh = self.screen.hh-2
		self.sList.yy =self.sList.yy+1
		self.sList:giveFocus()

    self.screen:size()
		push_screen_direct(self.screen)
end

function sDropdown:listAction(a,b)
		self.parentWidget.value  = b
		self.parentWidget.valuen = a
		self.parentWidget.rvalue  = b
		self.parentWidget.rvaluen = a
		self.parentWidget:change(a, b)
		remove_screen()
end

function sDropdown:change() end

function sDropdown:screenEscape()
		self.parentWidget.sList.sel = self.parentWidget.rvaluen
		self.parentWidget.value	= self.parentWidget.rvalue
		if current_screen() == self.parentWidget.screen then
				remove_screen()
		end
end

function sDropdown:paint(gc)
		gc:setColorRGB(255, 255, 255)
		gc:fillRect(self.x, self.y, self.w-1, self.h-1)

		gc:setColorRGB(0,0,0)
		gc:drawRect(self.x, self.y, self.w-1, self.h-1)

		if self.hasFocus then
        gc:setColorRGB(40, 148, 184)
        gc:drawRect(self.x-1, self.y-1, self.w+1, self.h+1)
        gc:setColorRGB(0, 0, 0)
    end

		gc:setColorRGB(192, 192, 192)
		gc:fillRect(self.x+self.w-21, self.y+1, 20, 19)
		gc:setColorRGB(224, 224, 224)
		gc:fillRect(self.x+self.w-22, self.y+1, 1, 19)

		gc:drawImage(self.img, self.x+self.w-18, self.y+9)

		gc:setColorRGB(0,0,0)
		local text = self.value
		if self.unitmode then
				text=text:gsub("([^%d]+)(%d)", numberToSub)
		end

		gc:drawString(textLim(gc, text, self.w-5-22), self.x+5, self.y, "top")
end
---**************************---
---**************************---
-- Start of new file
-- File: Builder/builder.lua
---**************************---
---**************************---

--[[
CircuitBuilder creates a Circuit from Schematic.elements data to be solved
This is a bit of a mess right now :( but it works. I will try to clean it up for the next update
]]

CircuitBuilder = class()

function CircuitBuilder:init(elements)
    self.circuit = Circuit()
    self.elements = elements
end

function CircuitBuilder:buildCircuit()
    self.grounds = {}
    self.nodes = {}
    self.attachments = {}
    self.connections = {}
    self.visited = {}
    for y, row in pairs(self.elements) do
        for x, e in pairs(row) do
            if e and e.kind == "Ground" then
                table.insert(self.grounds, e)
            elseif e and e.component then
                e.component.nodes[1] = false
                e.component.nodes[-1] = false
            end
        end
    end
    if #self.grounds < 1 then
        return
    end
    local gnd = self:Node(self.circuit, "gnd")
    table.insert(self.nodes, gnd)
    for i, v in pairs(self.grounds) do
        self:attached(v, gnd)
    end
    for i, v in pairs(self.attachments) do
        if not self.visited[self:toStr(v)] then
            local node = self:Node(self.circuit, "n"..#self.nodes)
            for j = -1, 1, 2 do
                if not v.component.nodes[j] then
                    node.node:connect(v.component, j)
                end
            end
            table.insert(self.nodes, node)
            self:attached(v, node)
            self.visited[self:toStr(v)] = true
        end
    end

    -- for i, v in pairs(self.nodes) do
    --     print(v.node.name .. ":"  .. #v.wire)
    --     for j, w in pairs(v.node.connections) do
    --         print(w.sign .. "*" .. w.component.name)
    --     end
    --     for j, w in pairs(v.node.voltageConnections) do
    --         print(w.sign .. "*" .. w.component.name)
    --     end
    -- end
    local solution = self.circuit:solve(self.nodes[1].node)
    for i, node in pairs(self.nodes) do
        for j, wire in pairs(node.wire) do
            wire.voltage = node.node.voltage
        end
    end
    if solution and solution ~= "false" then
        return true
    else
        return false
    end
end

function CircuitBuilder:attached(element, node)
    local a = element:attached()
    for sign, v in pairs(a) do
        for i, e in pairs(v) do
            if e.kind == "Wire" and not self.visited[self:toStr(e)] then
                table.insert(node.wire, e)
                self.visited[self:toStr(e)] = true
                self:attached(e, node)
            elseif e.component  then
                local side = e:isAttached(element.x, element.y)*(e.inverted and -1 or 1)
                if not self.visited[self:toStr(e, side)] then
                    table.insert(self.attachments, e)
                end
                if self.visited[self:toStr(e, -side)] then
                    self.visited[self:toStr(e)] = true
                end
                if not e.component.nodes[side] then
                    node.node:connect(e.component, side)
                end
                self.visited[self:toStr(e, side)] = true
            end
        end
    end
end

function CircuitBuilder:toStr(e, sign)
    return e.x .. ", " .. e.y .. (sign and "," .. sign or "")
end

function CircuitBuilder:Node(circuit, name)
    return {wire = {}, node = Node(circuit, name), attachments = {}}
end
---**************************---
---**************************---
-- Start of new file
-- File: Builder/widgets.lua
---**************************---
---**************************---

-------------------
-- Panel widget --
-------------------

InfoPanel = class(Widget)

function InfoPanel:init()
    self.ignoreFocus = true
    self.title = ""
    self.text =  {}
    self.visible = false
    self.dh = 100
    self.dw = 100
end

function InfoPanel:draw(gc)
    if self.visible then
        drawDialog(gc, self.title, self.x, self.y, self.w, 30 + #self.text*20)
        for i, text in pairs(self.text) do
            gc:drawString(text, self.x+4, self.y + 10 + 15*i, "top")
        end
    end
end

EditComponent = class(Dialog)

function EditComponent:init(element, x, y, w, h)
    Dialog.init(self, "Edit " .. element.kind, x, y, w, h)
    self.component = element.component
    self.parameter = element.parameter
    self.okayButton = sButton("Okay")
    self.okayButton.action = function()
        remove_screen(self)
    end
    self.nameLabel = sLabel("Name")
    self.valueLabel = sLabel(element.measure)
    self.nameInput = sInput()
    self.valueInput = sInput()
    self.nameInput.value = self.component.name
    self.valueInput.value = tostring(self.component[self.parameter].coeff)
    self.valueInput.number = true
    local units = {}
    for i, v in ipairs(Unit.prefixes) do
        units[i] = v .. self.component.unit
    end
    self.units = sDropdown(units)
		self.units.value = self.component[self.parameter]:unitStr()

    self:appendWidget(self.nameLabel, 4, 34)
    self:appendWidget(self.valueLabel, 4, 60)
    self:appendWidget(self.nameInput, 80, 34)
    self:appendWidget(self.valueInput, 80, 60)
    self:appendWidget(self.units, -4, 60)
    self:appendWidget(self.okayButton, -4, -4)
    self.valueInput:giveFocus()
end

function EditComponent:removed()
    self.component.name = self.nameInput.value:sub(0, 5)
    local prefix = string.sub(self.units.value, #self.units.value - #self.component.unit -  1, #self.units.value - #self.component.unit)
    self.component[self.parameter] = Unit(tonumber(self.valueInput.value), prefix, self.component.unit)
end


InfoWindow = class(Dialog)

function InfoWindow:init(title, lines, x, y, w, h)
    Dialog.init(self, title, x, y, w, h)
    self.okayButton = sButton("Okay")
    self.okayButton.action = function()
        remove_screen(self)
    end
    local longest = 0
    for i, v in pairs(lines) do
        local label = sLabel(v)
        self:appendWidget(label, 4, 10 + 15*i)
        longest = math.max(longest, #v)
    end
    -- self.h = 10 + 20*#lines
    -- self.w = 8*longest
    -- self.x = platform.window:width()/2 - self.w/2
    -- self.y = platform.window:height()/2 - self.h/2
    self.nameLabel = sLabel("Name")
    self:appendWidget(self.okayButton, -4, -4)
    self.okayButton:giveFocus()
end
---**************************---
---**************************---
-- Start of new file
-- File: Builder/element.lua
---**************************---
---**************************---

--[[
Elements are wrappers for components that handle GUI interactions
Attributes:
-- Component: Wrapped component
-- x, y: Location
-- kind: String name of the component
-- parameter: Which component variable is set by the edit dialog
-- measure: String representing the parameter
-- units: Unit type string
]]

Element = class()

-- movements in the grid that lead to an attached component, split by [element orientation][direction]
Element.touching = { vertical = { [-1] = {{0, -2},
                                          {0, -1},
                                          {-1, -1}},
                                  [1] =  {{0, 2},
                                          {-1, 1},
                                          {0, 1}}
                    },
                     horizontal = { [-1] = {{-1, 0},
                                            {0, 1},
                                            {0, -1}},
                                   [1] =   {{1, 0},
                                            {1, -1},
                                            {1, 1}}
                  }}

-- Statics ------------------------------
function Element.drawCircle(gc, x, y, size, horizontal)
    if horizontal then
        gc:drawLine(x, y, x + size/4, y)
        gc:drawArc(x + size/4, y - size/4, size/2, size/2, 0, 360)
        gc:drawLine(x + size*(3/4), y, x + size, y)
    else
        gc:drawLine(x, y, x, y + size/4)
        gc:drawArc(x - size/4, y + size/4, size/2, size/2, 0, 360)
        gc:drawLine(x, y + size*(3/4), x, y + size)
    end
end

-- Instance ------------------------------
function Element:init(schematic, inverted, x, y)
    self.schematic = schematic
    self.x = x
    self.y = y
    self.inverted = inverted
end

function Element:getDrawingConstants(grid, camera)
    return self.x*grid.size - camera.x,
           math.floor(self.y/2)*grid.size - camera.y,
           grid.size
end

function Element:draw(gc, grid, camera)
      x, y, size = self:getDrawingConstants(grid, camera)
      self.drawComponent(gc, x, y, size, self.inverted,  self.y % 2 == 0)
end

function Element:attached(req_sign)
    local r = {[1] = {}, [-1] = {}}
    local set = self.y % 2 == 0 and Element.touching.horizontal or Element.touching.vertical
    for sign, v in pairs(set) do
        if (not req_sign or req_sign == sign) then
            for i, e in pairs(v) do
                local x, y = self.x + e[1], self.y + e[2]
                local a = self.schematic.elements[y][x]
                if a then
                    if oneWay or a:isAttached(self.x, self.y) then
                        table.insert(r[sign], a)
                    end
                end
            end
        end
    end
    return r
end

function Element:isAttached(x, y, req_sign)
      local set = self.y % 2 == 0 and Element.touching.horizontal or Element.touching.vertical
      for sign, v in pairs(set) do
          if (not req_sign or req_sign == sign) then
              for i, e in pairs(v) do
                  local x1, y1 = self.x + e[1], self.y + e[2]
                  if x1 == x and y1 == y then
                      return sign
                  end
              end
          end
      end
      return false
end

------------------
-- ElementWire  --
------------------

ElementWire = class(Element)

-- Statics ------------------------------

ElementWire.kind = "Wire"
ElementWire.tool = "wire"

function ElementWire.drawComponent(gc, x, y, size, inverted, horizontal)
    if horizontal then
        gc:fillRect(x, y, size, 1)
    else
        gc:fillRect(x, y, 1, size)
    end
end

-- Instance ------------------------------
function ElementWire:init(schematic, inverted, x, y, name, value)
    Element.init(self, schematic, inverted, x, y)
    self.voltage = false
end

function ElementWire:getDialogInfo()
    if not self.voltage then
        return {"??? V"}
    else
        return {Unit.pretty(self.voltage) .. "V"}
    end
end

function ElementWire:getDialogTitle()
    return "Wire"
end

----------------------
-- ElementResistor  --
----------------------

ElementResistor = class(Element)

-- Statics ------------------------------
ElementResistor.kind = "Resistor"
ElementResistor.tool = "resistor"
ElementResistor.measure = "Resistance"
ElementResistor.parameter = "resistance"

function ElementResistor.drawComponent(gc, x, y, size, inverted, horizontal)
    local zigzag = {0, 0, -size/5, 0, size/5, 0, -size/5, 0, size/5, 0, 0}
    if inverted then
         reverse(zigzag)
    end
    if horizontal then
        for i = 0, 9 do
            gc:drawLine(x + size*(i/10), y + zigzag[i+1], x + size*((i+1)/10), y + zigzag[i+2])
        end
    else
        for i = 0, 9 do
            gc:drawLine(x + zigzag[i+1], y + size*(i/10), x + zigzag[i+2], y + size*((i+1)/10))
        end
    end
end

-- Instance ------------------------------
function ElementResistor:init(schematic, inverted, x, y, name, value, unit)
    Element.init(self, schematic, inverted, x, y)
    self.component = Resistor(schematic.circuit, name, false, false, value, unit)
end

function ElementResistor:getDialogInfo()
    local voltage, current = "???", "???"
    if self.component.nodes[1] and self.component.nodes[-1] then
        if self.component.nodes[1].voltage and self.component.nodes[-1].voltage then
            voltage = math.abs(self.component.nodes[1].voltage - self.component.nodes[-1].voltage)
            current = Unit.pretty(voltage/self.component.resistance:value())
            voltage = Unit.pretty(voltage)
        end
    end
		return {self.component.resistance:str(),
						"-----",
						"V → " .. voltage .. "V",
						"I → " .. current .. "A"}
end

function ElementResistor:getDialogTitle()
		return "Resistor " .. self.component.name
end
--------------------
-- ElementVoltage --
--------------------

ElementVoltage = class(Element)

-- Statics ------------------------------
ElementVoltage.kind = "Voltage Source"
ElementVoltage.tool = "voltage"
ElementVoltage.measure = "Voltage"
ElementVoltage.parameter = "voltage"

function ElementVoltage.drawComponent(gc, x, y, size, inverted, horizontal)
    Element.drawCircle(gc, x, y, size, horizontal)
    -- relative to start coordinates, written for vertical
    local lines = {{size*(1/10), size*(2/5), -size*(1/10), size*(2/5)},
                  {size*(1/10), size*(3/5), -size*(1/10), size*(3/5)},
                  {0, size*((inverted and 7 or 3)/10), 0, size*(1/2)}}
    for i, line in ipairs(lines) do
         if horizontal then
             gc:drawLine(x + line[2], y + line[1], x + line[4], y + line[3])
         else
             gc:drawLine(x + line[1], y + line[2], x + line[3], y + line[4])
         end
    end
end

-- Instance ------------------------------
function ElementVoltage:init(schematic, inverted, x, y, name, value)
    Element.init(self, schematic, inverted, x, y)
    self.component = VoltageSource(schematic.circuit, name, false, false, value, unit)
end

function ElementVoltage:getDialogInfo()
		return {self.component.voltage:str()}
end

function ElementVoltage:getDialogTitle()
		return "Source " .. self.component.name
end

--------------------
-- ElementCurrent --
--------------------

ElementCurrent = class(Element)

-- Statics ------------------------------
ElementCurrent.kind = "Current Source"
ElementCurrent.tool = "current"
ElementCurrent.measure = "Current"
ElementCurrent.parameter = "current"

function ElementCurrent.drawComponent(gc, x, y, size, inverted, horizontal)
    Element.drawCircle(gc, x, y, size, horizontal)
    -- relative to start coordinates, written for vertical
    local lines = {{0, size*(1/3), 0, size*(2/3)},
                  {size/6, size*(1/2), 0, size*((inverted and 1 or 2)/3)},
                  {-size/6, size*(1/2), 0, size*((inverted and 1 or 2)/3)}}
    for i, line in ipairs(lines) do
         if horizontal then
             gc:drawLine(x + line[2], y + line[1], x + line[4], y + line[3])
         else
             gc:drawLine(x + line[1], y + line[2], x + line[3], y + line[4])
         end
    end
end

-- Instance ------------------------------
function ElementCurrent:init(schematic, inverted, x, y, name, value)
    Element.init(self, schematic, inverted, x, y)
    self.component = CurrentSource(schematic.circuit, name, false, false, value, unit)
end

function ElementCurrent:getDialogInfo()
    local voltage = "???"
    if self.component.nodes[1] and self.component.nodes[-1] then
        if self.component.nodes[1].voltage and self.component.nodes[-1].voltage then
            voltage = Unit.pretty(math.abs(self.component.nodes[1].voltage - self.component.nodes[-1].voltage))
        end
    end
		return {self.component.current:str(),
						"-----",
						"V → " .. voltage .. "V"}
end

function ElementCurrent:getDialogTitle()
		return "Source " .. self.component.name
end

-------------------
-- ElementGround --
-------------------

ElementGround = class(Element)

-- Statics ------------------------------
ElementGround.kind = "Ground"
ElementGround.tool = "ground"

function ElementGround.drawComponent(gc, x, y, size, inverted, horizontal)
    -- relative to start coordinates, written for vertical
    local lines = {{0, 0, 0, size*(1/6)},
                  {-size/3, size*(1/6), size/3, size*(1/6)},
                  {-size/6, size*(2/6), size/6, size*(2/6)},
                  {-size/9, size*(3/6), size/9, size*(3/6)}}

    for i, line in ipairs(lines) do
         if inverted then
            line = {line[1], size - line[2], line[3], size - line[4]}
         end
         if horizontal then
             gc:drawLine(x + line[2], y + line[1], x + line[4], y + line[3])
         else
             gc:drawLine(x + line[1], y + line[2], x + line[3], y + line[4])
         end
    end
end

-- Instance ------------------------------
function ElementGround:init(schematic, inverted, x, y, name, value, unit)
    Element.init(self, schematic, inverted, x, y)
end

function ElementGround:getDialogInfo()
		return {""}
end

function ElementGround:getDialogTitle()
		return "Ground"
end


function ElementGround:attached()
    return Element.attached(self, self.inverted and 1 or -1)
end

function ElementGround:isAttached(x, y)
    return Element.isAttached(self, x, y, self.inverted and 1 or -1)
end
---**************************---
---**************************---
-- Start of new file
-- File: Builder/schematic.lua
---**************************---
---**************************---

--[[
Schematic implements the GUI used to build circuits
Attributes:
-- elements: 2D representation of grid elementations
             rows alternate between horizontal and vertical elements
              { _  _  _  _ },
              {|| || || || },
              { _  _  _  _ }
-- panel: Reference to the info panel dialog
-- theme: Color theme:
-- selection: {x, y} location of selector
-- camera: Pixel shift of the schematic view
-- selected: Currently selected element
-- tool: Currently active tool
]]

Schematic = class(WWidget)

Schematic.defaultGrid = {width = 10, height = 10, size = 50}

Schematic.defaultTheme = {
    textColor         = {000, 000, 000},
    backgroundColor   = {255, 255, 255},
    borderColor       = {150, 150, 150},
    gridColor         = {235, 235, 235},
    unplacedColor     = {150, 150, 150},
    placedColor       = {050, 050, 255},
    selectorColor     = {200, 200, 200},
    selectedColor     = {255, 050, 050}
}

Schematic.creatorTools = {["wire"] = {element = ElementWire, name = "W", editor = false},
                          ["resistor"] = {element = ElementResistor, name = "R", editor = true},
                          ["voltage"] = {element = ElementVoltage, name = "V", editor = true},
                          ["current"] = {element = ElementCurrent, name = "C", editor = true},
                          ["ground"] = {element = ElementGround, name = "G", editor = false}
                         }

function Schematic:init(arg)
    Widget.init(self)

    -- Window Settings
    self.panel = arg.panel
    self.indicator = arg.indicator
    self.dw = 100
    self.dh = 100
    self.theme = arg.theme or Schematic.defaultTheme

    -- Grid / Circuit
    self.grid = arg.grid or self.defaultGrid
    self.grid.height = self.grid.height*2
    self.circuit = Circuit()

    -- Elements and Properties
    self.elements = {}
    self.toolShortcuts = {}
    self.camera = {x = 0, y = 0}
    self.selection = {x = 1, y = 1}
    self.selected = false
    self.tool = "resistor"
    self.inverted = true

    -- Initialize Elements
    self:clearCircuit()

    -- self:testCircuit()
end


function Schematic:clearCircuit()
    for y = 0, self.grid.height*2 do
        self.elements[y] = {}
        for x = 0, self.grid.width + (y % 2 == 0 and -1 or 0) do
            table.insert(self.elements[y], false)
        end
    end
end

function Schematic:draw(gc)

    gc:clipRect("set", self.x, self.y, self.w, self.h)

    -- Background
    gc:setColorRGB(unpack(self.theme.backgroundColor))
    gc:fillRect(self.x, self.y, self.w, self.h)
    self:drawGrid(gc)

    -- Border
    gc:setColorRGB(unpack(self.theme.borderColor))
    gc:drawRect(self.grid.size - self.camera.x, self.grid.size - self.camera.y,
                self.grid.width*self.grid.size-self.grid.size, self.grid.height*self.grid.size/2)

    -- Details
    self:drawElements(gc)
    self:drawSelector(gc)

    gc:clipRect("reset")

    self:updatePanel()

end

function Schematic:drawElements(gc)
    for y, row in pairs(self.elements) do
        for x, e in pairs(row) do
            if e then
                gc:setColorRGB(unpack(self:getColor(e.x, e.y)))
                e:draw(gc, self.grid, self.camera)
            end
        end
    end
end

function Schematic:getColor(x, y)
    if self.tool == "inspector" and self.selected then
        if x == self.selected.x and y == self.selected.y then
            return self.theme.selectedColor
        end
    end
    return self.theme.placedColor
end

function Schematic:drawGrid(gc)
    gc:setColorRGB(unpack(self.theme.gridColor))
    local s = self.grid.size
    local start = {x = self.x + ((self.w / 2) % s), y = self.y + ((self.h / 2) % s)}
    for i = 0, math.ceil(self.h / s) do
          gc:drawRect(0, i * s - (self.camera.y % s), self.w, 1)
    end
    for i = 0, math.ceil(self.w / s) do
        gc:drawRect(i * s - (self.camera.x % s), 0, 1, self.h)
    end
end

function Schematic:drawSelector(gc)
    -- Draw general selector
    gc:setColorRGB(unpack(self.theme.selectorColor))
    local x, y = self:locFromIndex(self.selection.x, self.selection.y)
    gc:drawArc(x, y, self.grid.size, self.grid.size, 0, 360)

    -- Draw Specific Tool
    gc:setColorRGB(unpack(self.theme.unplacedColor))
    local horizontal = self.selection.y % 2 == 0
    if horizontal then
        y = y + self.grid.size/2
    else
        x = x + self.grid.size/2
    end
    if Schematic.creatorTools[self.tool] then
        Schematic.creatorTools[self.tool].element.drawComponent(gc, x, y, self.grid.size, self.inverted, horizontal)
    end

    if self.tool == "loop" and self.loopStart then
        local x1, y1 = self:locFromIndex(self.selection.x, self.selection.y)
        local x2, y2 = self:locFromIndex(self.loopStart.x, self.loopStart.y)
        gc:setColorRGB(unpack(self.theme.selectedColor))
        gc:drawRect(math.min(x1, x2) + self.grid.size/2, math.min(y1, y2) + self.grid.size/2,
                    math.abs(x1-x2), math.abs(y1-y2))
    else
        self.loopStart = false
    end
end

function Schematic:locFromIndex(x, y)
    return x*self.grid.size - self.camera.x,
               (y/2)*self.grid.size + self.grid.size/2 - self.camera.y
end

function Schematic:updatePanel()
    local selected = self.selected
    self.panel.visible = selected
    if selected then
        local x, y, size = Element.getDrawingConstants(selected, self.grid, self.camera)
        if  x + size/2 > self.w/2 then
            self.panel.xx = 10
        else
            self.panel.xx = -10
        end
        self.panel.title = selected:getDialogTitle()
        self.panel.text = selected:getDialogInfo()
    end
end

function Schematic:insertComponent(component, x, y, ...)
    self.elements[y][x] = component(self, self.inverted, x, y, ...)
    return self.elements[y][x]
end

function Schematic:testCircuit()
    self:insertComponent(ElementWire, 2, 7)
    self:insertComponent(ElementVoltage, 2, 9, "V1", 10)
    self:insertComponent(ElementCurrent, 2, 6, "I1", 10)
    self:insertComponent(ElementWire, 3, 6)
    self:insertComponent(ElementResistor, 4, 6, "R1", 10)
    self:insertComponent(ElementWire, 5, 6)
    self:insertComponent(ElementResistor, 6, 7, "R2", 10)
    self:insertComponent(ElementWire, 6, 9)
    self:insertComponent(ElementWire, 2, 10)
    self:insertComponent(ElementWire, 3, 10)
    self:insertComponent(ElementWire, 4, 10)
    self:insertComponent(ElementWire, 5, 10)
end

function Schematic:zoom(zoom)
    self.grid.size = self.grid.size + zoom
    self.grid.size = math.max(math.min(self.grid.size, 60), 20)
    local x, y = self:locFromIndex(self.selection.x, self.selection.y)
    self.camera.x = self.camera.x - (self.w/2 - x)/2
    self.camera.y = self.camera.y - (self.h/2 - y)/2
    platform.window:invalidate()
end

function Schematic:moveSelection(x, y)
    local new = {x = self.selection.x + x, y = self.selection.y + y}
    self.selection.x = math.min(math.max(0.5, new.x), self.grid.width-0.5)
    self.selection.y = math.min(math.max(0, new.y), self.grid.height)
    self:selectAt(self.selection.x, self.selection.y)
    self:pan()
    platform.window:invalidate()
end

function Schematic:pan()
    local x, y = self:locFromIndex(self.selection.x, self.selection.y)
    if x > self.w - self.grid.size then
        self.camera.x = self.selection.x*self.grid.size - self.w + self.grid.size
    end
    if x < 0 then
        self.camera.x = self.selection.x*self.grid.size
    end
    if y > self.h - self.grid.size then
        self.camera.y = ((self.selection.y)/2)*self.grid.size - self.h + 1.5*self.grid.size
    end
    if y < 0 then
        self.camera.y = ((self.selection.y)/2)*self.grid.size + self.grid.size/2
    end
end

function Schematic:indexFromCoord(x, y)
    local x, y = x, y + 2
    if y % 2 ~= 0 then
        x = x + 0.5
    end
    return x, y
end

function Schematic:selectAt(x, y)
    local x, y = self:indexFromCoord(x, y)
    self.selected = false
    if self.elements[y] and self.elements[y][x] then
        self.selected = self.elements[y][x]
    end
end

function Schematic:onMouseDown()
    self.meDown = true
end

function Schematic:charIn(char)
    local arrow = {["4"] = "left",
                   ["2"] = "down",
                   ["6"] = "right",
                   ["8"] = "up",
    }
    if char == '-' then
        self:zoom(-5)
    elseif char == "+" or char == "=" then
        self:zoom(5)
    elseif char == "f" or char == "5" then
        self:invert()
    elseif arrow[char] then
        self:arrowKey(arrow[char])
    elseif char == "s" then
        self:solve()
    elseif self.toolShortcuts[char] then
        self.tool = self.toolShortcuts[char]
    end
end

function Schematic:invert()
    if self.tool == "inspector" and self.selected then
        self.selected.inverted = not self.selected.inverted
        self:edit()
    end
    self.inverted = not self.inverted
end

function Schematic:solve()
    local cb = CircuitBuilder(self.elements)
    if cb:buildCircuit() then
        self.indicator.color = {0, 255, 0}
        self.indicator.text = "✓"
    end
    platform.window:invalidate()
end

function Schematic:edit()
    self.indicator.color = {255, 0, 0}
    self.indicator.text = "≟"
end

function Schematic:arrowKey(arrow)
    local dir = {["left"] = {-0.5, 0},
                    ["down"] = {0, 1},
                    ["right"] = {0.5, 0},
                    ["up"] = {0, -1},
                   }
     if dir[arrow] then
         self:moveSelection(dir[arrow][1], dir[arrow][2])
     end
end

function Schematic:clearKey()
    self:clearCircuit()
    platform.window:invalidate()
end

function Schematic:mouseUp(x, y)
    self:enterKey()
end

function Schematic:rightMouseUp(x, y)
    self:backspaceKey()
end

function Schematic:enterKey()
    self:edit()
    if self.tool == "inspector" and self.selected then
        if self.creatorTools[self.selected.tool] and self.creatorTools[self.selected.tool].editor then
            self:launchEditor(self.selected)
        end
    elseif self.tool == "loop" then
        local x, y = self:indexFromCoord(self.selection.x, self.selection.y)
        if x % 1 ~= 0 then
            if not self.loopStart then
                self.loopStart = {x = self.selection.x, y = self.selection.y}
            else
                local x1, y1 = math.min(self.selection.x, self.loopStart.x), math.min(self.selection.y, self.loopStart.y)
                local x2, y2 = math.max(self.selection.x, self.loopStart.x), math.max(self.selection.y, self.loopStart.y)
                self:makeWireLoop(x1, y1, x2, y2)
                self.loopStart = false
            end
        end
    elseif Schematic.creatorTools[self.tool] then
        local x, y = self:indexFromCoord(self.selection.x, self.selection.y)
        if x % 1 == 0 then
            local c = self:insertComponent(Schematic.creatorTools[self.tool].element, x, y,
                                           Schematic.creatorTools[self.tool].name, 1, "")
            self.selected = c
            if Schematic.creatorTools[self.tool].editor then
                self:launchEditor(c)
            end
        end
    end
end

function Schematic:makeWireLoop(x1, y1, x2, y2)
    for x = x1, x2, .5 do
        for y = y1, y2 do
            if x == x1 or x == x2 or y == y1 or y == y2 then
                local wx, wy = self:indexFromCoord(x, y)
                if wx % 1 == 0 then
                    self:insertComponent(ElementWire, wx, wy, "w")
                end
            end
        end
    end
end

function Schematic:backspaceKey()
    local x, y = self:indexFromCoord(self.selection.x, self.selection.y)
    if x % 1 == 0 then
        self.elements[y][x] = false
        self.selected = false
    end
end

function Schematic:launchEditor(element)
    local editor = EditComponent(element, 20, 20, -40, -40)
    push_screen_direct(editor)
end

function Schematic:mouseMove(x, y)
    local cx, cy = self.camera.x, self.camera.y
    local mx, my = (x + cx - self.grid.size/4)/self.grid.size, (y + cy - self.grid.size/4)/(self.grid.size/2)
    self:moveSelection(math.floor((mx - self.selection.x)*2)/2,
                                 math.floor((my - self.selection.y - 1)))
end
---**************************---
---**************************---
-- Start of new file
-- File: Solver/units.lua
---**************************---
---**************************---

Unit = class()

-- prefixes ordered by magnitude, for dropdown reference
Unit.prefixes =      { "m", "",  "k"}
Unit.prefix_values = {["m"] = -3, [""] = 0, ["k"] = 3}

function Unit:init(coeff, prefix, unit)
    self.coeff = coeff or 0
    self.prefix = prefix or ""
    self.unit = unit or ""
end

function Unit.pretty(x)
    local u = ""
    local s = 0
    for i, v in pairs(Unit.prefixes) do
        local p = Unit.prefix_values[v]
        if p < 0 then
            if math.abs(x) < math.pow(10, p)*100 then
                x = x * math.pow(10, -p)
                u = v
                break
            end
        else
            if math.abs(x) >= math.pow(10, p)/10 then
                x = x * math.pow(10, -p)*math.pow(10, s)
                s = s + p
                u = v
            end
        end
    end
    x = math.floor(x*100 + 0.5)/100
    return string.sub(tostring(x), 0, 7) .. u
end

function Unit:value()
    return self.coeff * math.pow(10, Unit.prefix_values[self.prefix])
end

function Unit:str()
    return self.coeff .. self.prefix .. self.unit
end

function Unit:unitStr()
    return self.prefix .. self.unit
end
---**************************---
---**************************---
-- Start of new file
-- File: Solver/circuit.lua
---**************************---
---**************************---

--include "component.lua"
--include "node.lua"

--[[
Circuit Objects store Nodes and Components and solve for their respective unknowns
Attributes:
-- Nodes: A list of nodes in the circuit
-- Components: A list of components in the circuit
]]

Circuit = class()
	-- Circuit Constructor
function Circuit:init()
		self.nodes = {}
		self.components = {}
end
-- Solve the circuit given a node tied to ground
-- Sets voltage / current parameters for all components
---- as well as voltage parameters for every node
function Circuit:solve(ground)
		local eq = 'solve(system('
		for i, node in pairs(self.nodes) do
				if #node.voltageConnections > 0 then
						eq = eq .. node:getVoltageEq()
						eq = eq .. node:getSuperEq({[node.name] = true})
				else
						eq = eq .. node:getCurrentEq()
				end
		end
		local eval = eq .. ground.name .. ' = 0' .. '), {' .. self:getNodeList() .. '})'
		local ans = math.evalStr(eval)
		print(eval)
		print(ans)
		if ans then
				local solutions = split(ans, " and ")
				for i, solution in pairs(solutions) do
						local parts = split(solution, "=")
						for j, v in pairs(self.nodes) do
								if v.name == parts[1] then
										local number = string.gsub(math.evalStr(parts[2] .. "▶Decimal"), "%−", "-")
										parts = split(number, "")
										parts[2] = parts[2] or 0
										print(parts[2])
										v.voltage = tonumber(parts[1])*math.pow(10, tonumber(parts[2]))
										print(v.voltage)
								end
						end
				end
		end
		return ans
end

function split(s, delimiter)
    result = {};
    for match in (s..delimiter):gmatch("(.-)"..delimiter) do
        table.insert(result, match);
    end
    return result;
end

-- Returns a list of node names seperated by commas
function Circuit:getNodeList()
		local nodeList = ''
		for i, v in pairs(self.nodes) do
				nodeList = nodeList .. v.name .. ', '
		end
		if #nodeList > 0 then
		 		nodeList = string.sub(nodeList, 0, #nodeList-2)
		end
		return nodeList
end

Connection = class()
	-- Connection Constructor
	-- Used to store connected components and the side they are connected by
function Connection:init(component, sign)
		self.component = component
		self.sign = sign
end
---**************************---
---**************************---
-- Start of new file
-- File: Solver/node.lua
---**************************---
---**************************---

--[[
Nodes are ares of the circuit that are directly connected by wire
Attributes:
-- Circuit: Circuit the component is part of
-- Name: A name for the component
-- voltage: Voltage at the node (based on a grounded node used when solving)
-- connections: passive components and current sources connected
-- voltageConnections: voltage sources connected
NOTE: connections and voltageConnections are seperated because nodal analysis
      is used to solve these circuits.
]]

Node = class()

-- Nodes Constructor
function Node:init(circuit, name)
		self.circuit = circuit
		table.insert(self.circuit.nodes, self)
		self.connections = {}
		self.voltageConnections = {}
		self.voltage = false
		self.name = name
end

-- Connect a component to a node
-- (sign is 1 or -1, and is the side of component connected)
function Node:connect(component, sign)
		component.nodes[sign] = self
		if component.getVoltage then
				table.insert(self.voltageConnections, Connection(component, sign))
		else
				table.insert(self.connections, Connection(component, sign))
		end
end

-- Add together outgoing currents of connected nodes
-- Used in both Current Equation and Super Node Equation
function Node:addCurrents()
		local eq = ''
		for i, v in pairs(self.connections) do
				eq = eq .. v.component:getCurrent(v.sign) .. ' + '
		end
		return eq
end

-- Get the Current Equation
-- Used when all nodes are connected by a resistive or current source
function Node:getCurrentEq()
		local eq = '0 = 0 + ' .. self:addCurrents()
		eq = string.sub(eq, 0, #eq-2) .. ', '
		return eq
end

-- Get the Voltage Equation
-- Returns equations relating to nodes connected by a voltage source
function Node:getVoltageEq()
		local eq = ''
		for i, v in pairs(self.voltageConnections) do
				eq = eq .. v.component:getVoltage(v.sign) .. ', '
		end
		return eq
end

-- Get Super Node Equation
-- This links all the outgoing equations of nodes connected by a voltage source
function Node:getSuperEq(visited)
		local eq = ''
		local og = false
		if #visited == 1 then
				eq = eq .. '0 = 0 + '
				local og = true
		end
		eq = eq .. self:addCurrents()
		for i, v in pairs(self.voltageConnections) do
				if not visited[v.component:getNode(-v.sign).name]  then --not visited[v.component.name] then
					 -- visited[v.component.name] = true
					 	visited[v.component:getNode(-v.sign).name] = true
						eq = eq .. v.component:getNode(-v.sign):getSuperEq(v.component, visited)
				end
		end
		if og then
				eq = string.sub(eq, 0, #eq-2) .. ', '
		end
		return eq
end
---**************************---
---**************************---
-- Start of new file
-- File: Solver/component.lua
---**************************---
---**************************---

--[[
Components are devices used to connect nodes
Attributes:
-- Circuit: Circuit the component is part of
-- Name: A name for the component
-- nodes[1]: Node connected to the positive end of the component
-- nodes[-1]: Node connected to the negative end of the component
-- current: Current through the component (set when circuit is solved)
-- voltage: Voltage across the component (set when circuit is solved)
]]

Component = class()
-- Component Constructor
-- Takes the circuit it is part of, a name, and the connected nodes
function Component:init(circuit, name, posNode, negNode)
		self.circuit = circuit
		table.insert(self.circuit.components, self)
		self.name = name
		self.nodes = {}
		self.nodes[1] = posNode
		self.nodes[-1] = negNode
		if posNode and negNode then
				posNode:connect(self, 1)
				negNode:connect(self, -1)
		end
		self.current = Unit()
		self.voltage = Unit()
end

-- Get the node connected to a given side of the component
function Component:getNode(sign)
		return self.nodes[sign]
end

Resistor = class(Component)
Resistor.unit = "Ω"
	-- Resistor
	-- Takes in a resistance in Ohms
function Resistor:init(circuit, name, posNode, negNode, resistance, unit)
		Component.init(self, circuit, name, posNode, negNode)
		self.resistance = Unit(resistance, unit, self.unit)
end

-- Current is (v2-v1)/R
function Resistor:getCurrent(node)
		local c = '(' .. self.nodes[-node].name .. '-' .. self.nodes[node].name .. ')'
		c = c .. '/' .. self.resistance:value()
		return c
end

VoltageSource = class(Component)
VoltageSource.unit = "V"
-- Voltage Source
-- Takes in a voltage in Volts
function VoltageSource:init(circuit, name, posNode, negNode, voltage, unit)
		Component.init(self, circuit, name, posNode, negNode)
		self.voltage = Unit(voltage, unit, self.unit)
end

-- Voltage at given node = Voltage at opposite node + Voltage from source
function VoltageSource:getVoltage(node)
		return self.nodes[-node].name .. '=' .. self.nodes[node].name .. ' + ' .. self.voltage:value()*node
end


CurrentSource = class(Component)
CurrentSource.unit = "A"
-- Current Source
-- Takes in a current in Amperes
function CurrentSource:init(circuit, name, posNode, negNode, current, unit)
		Component.init(self, circuit, name, posNode, negNode)
		self.current = Unit(current, unit, self.unit)
end

-- Current from this device is simply its supplied current
function CurrentSource:getCurrent(node)
		return self.current:value()*node
end
---**************************---
---**************************---
-- Start of new file
-- File: main.lua
---**************************---
---**************************---

function main()

    local main = WScreen()
    local panel = InfoPanel()
    local indicator = sLabel("")
    panel:setSize(100, 100)
    local workspace = Schematic{panel = panel, indicator = indicator}
    workspace:setSize(-1, -1)
    main:appendWidget(workspace, 0, 0)
    main:appendWidget(panel, -10, 10)
    main:appendWidget(indicator, -5, -5)
    workspace:giveFocus()

    function main:paint(gc)
        gc:setColorRGB(55, 55, 55)
        gc_clipRect(gc, "set", 0, 0, platform.window:width(), platform.window:height())
        gc:fillRect(0, 0, platform.window:width(), platform.window:height())
    end

    function main:tabKey()
        circuitThings()
    end

    function about ()
        local aboutText = {"NSPICE is a circuit design and simulation",
                           "program developed by M4dasOne",
                           "",
                           "GUI Toolkit by Adrien 'Adriweb' Betrand and",
                           "Jim Bauwens",
                           "",
                           "Special Thanks: Inspired-lua.org, TIPlanet.org",
                           "License: LGPL3"}
        local iw = InfoWindow("About NSPICE v" .. nspice_version, aboutText, 10, 10, -20, -20)
        push_screen_direct(iw)
    end

    function help ()
        local helpText = { "Be sure to connect all circuits to Ground",
                           "before solving!",
                           "",
                           "Controls",
                           "Arrows / Numpad / Mouse: Navigate",
                           "Enter / Mouse: Insert / Edit",
                           "Del: Delete",
                           "+ / - : Zoom",
                           "f / 5 : Flip component"}
        local iw = InfoWindow("Help", helpText, 10, 10, -20, -20)
        push_screen_direct(iw)
    end

    local tools = {{menuIndex = "Inspector",      tool = "inspector",   shortcut = "i"},
                   {menuIndex = "Wire Segment",   tool = "wire",        shortcut = "w"},
                   {menuIndex = "Wire Loop",      tool = "loop",        shortcut = "l"},
                   {menuIndex = "Resistor",       tool = "resistor",    shortcut = "r"},
                   {menuIndex = "Voltage Source", tool = "voltage",     shortcut = "v"},
                   {menuIndex = "Current Source", tool = "current",     shortcut = "c"},
                   {menuIndex = "Ground",         tool = "ground",      shortcut = "g"}}

    local menu = {
        {"Circuit",
            {"Solve (s)", function() workspace:solve() end},
            {"Clear (ctrl+del)", function() workspace:clearKey() end}
        },
        {"Tools"
        },
        {"Help",
            {"Help", help},
            {"About", about},
        }
    }

    local toolLookup = {}

    function setTool (parent, option)
        workspace.tool = toolLookup[option]
    end

    for i, v in ipairs(tools) do
        local m = v.menuIndex
        if v.shortcut then
            m = m .. " (" .. v.shortcut .. ")"
            workspace.toolShortcuts[v.shortcut] = v.tool
        end
        table.insert(menu[2], {m, setTool})
        toolLookup[m] = v.tool
    end

    toolpalette.register(menu)
    push_screen(main)

end

main()
